// Fill out your copyright notice in the Description page of Project Settings.

#include "Building.h"
#include <algorithm>
#include "Segment.h"

Building::Building()
{
}

Building::Building(TArray<FVector2D> _layout, float _height, EBuildingType _buildingType)
{
	layout = _layout;
	height = _height;
	buildingType = _buildingType;
}

Building::~Building()
{
}

Building Building::Split(FVector2D firstSplitLocation, FVector2D secondSplitLocation, int firstSplitIndex, int secondSplitIndex, bool createNewBuilding, float minSize, FVector2D offset)
{
	TArray<FVector2D> newLayout = layout;
	//check for building big enough here for new one and old one @ StreetGenerator.cpp at update and construct buildings
	if (layout[firstSplitIndex] != firstSplitLocation) {
		layout.Insert(firstSplitLocation + offset, firstSplitIndex + 1);
		layout.RemoveAt(firstSplitIndex + 2);
		if(layout[(secondSplitIndex + 1) % newLayout.Num()] != secondSplitLocation)	layout.Insert(secondSplitLocation + offset, secondSplitIndex + 1);
		for (int i = 0; i < secondSplitIndex - firstSplitIndex - 1; i++)
		{
			layout.RemoveAt(secondSplitIndex - i);
		}
	}
	else {
		if (layout[(secondSplitIndex + 1) % layout.Num()] != secondSplitLocation) {
			layout.Insert(secondSplitLocation + offset, secondSplitIndex + 1);
		}
		layout.RemoveAt(firstSplitIndex + 1);
	}

	if (createNewBuilding) {
		int length = newLayout.Num() - 1;
		//bool secures to get correct polygon if index goes over the length ex. secondSplitIndex = 3 and to remove 3 elements -> would remove in third step index 1, would have been correct the previous step but now is index = 0
		bool removeCheck = false;
		if (newLayout[firstSplitIndex] != firstSplitLocation) {
			newLayout.Insert(firstSplitLocation - offset, firstSplitIndex + 1);
			newLayout.RemoveAt(firstSplitIndex);
		}
		if (newLayout[(secondSplitIndex + 1) % newLayout.Num()] != secondSplitLocation) {
			newLayout.Insert(secondSplitLocation - offset, secondSplitIndex + 1);
			for (int j = 0; j < firstSplitIndex - secondSplitIndex + length; j++)
			{
				if (!removeCheck && ((secondSplitIndex + 2) % newLayout.Num()) == 0) removeCheck = true;
				if (removeCheck) newLayout.RemoveAt(0);
				else newLayout.RemoveAt((secondSplitIndex + 2) % newLayout.Num());
			}
		}
		else {
			for (int j = 0; j < firstSplitIndex - secondSplitIndex + length - 1; j++)
			{
				newLayout.RemoveAt((secondSplitIndex + 2) % newLayout.Num());
			}
		}

		Building splitPart = Building(newLayout, height, buildingType);
		if (splitPart.GetPolygonArea() > minSize) return splitPart;
	}
	return Building();
}

bool Building::OnSegment(FVector2D p, FVector2D q, FVector2D r)
{
	if (q.X <= std::max(p.X, r.X) && q.X >= std::min(p.X, r.X) &&
		q.Y <= std::max(p.Y, r.Y) && q.Y >= std::min(p.Y, r.Y))
		return true;

	return false;
}

int Building::Orientation(FVector2D p, FVector2D q, FVector2D r)
{
	// https://www.geeksforgeeks.org/orientation-3-ordered-points/ 
	float val = (q.Y - p.Y) * (r.X - q.X) -
		(q.X - p.X) * (r.Y - q.Y);

	if (val == 0) return 0;  // colinear 

	return (val > 0) ? 1 : 2; // clock or counterclock wise 
}

bool Building::CheckForIntersection(Building other)
{
	int mod1 = layout.Num();
	int mod2 = other.layout.Num();
	for (int i = 0; i < mod1; i++)
	{
		for (int o = 0; o < mod2; o++)
		{
			if (CheckIntersection(layout[i], layout[(i + 1) % mod1], other.layout[o], other.layout[(o + 1) % mod2])) return true;
			//check if one building is inside of the other
			int c1 = Segment(other.layout[o], other.layout[(o + 1) % mod2], false).CheckStreetEndInside(Building(layout, height, buildingType));
			int c2 = Segment(layout[i], layout[(i + 1) % mod1], false).CheckStreetEndInside(Building(other.layout, other.height, other.buildingType));
			if ((c1 == 3 && c2 == 0) || (c1 == 0 && c2 == 3)) return true;
		}
	}
	return false;
}

float Building::GetLowY() {
	float lowest = layout[0].Y;
	for (int i = 0; i < layout.Num(); i++)
	{
		lowest = fminf(lowest, layout[i].Y);
	}
	return lowest;
}

float Building::GetLowX() {
	float lowest = layout[0].X;
	for (int i = 0; i < layout.Num(); i++)
	{
		lowest = fminf(lowest, layout[i].X);
	}
	return lowest;
}

float Building::GetHighY() {
	float highest = layout[0].Y;
	for (int i = 0; i < layout.Num(); i++)
	{
		highest = fmaxf(highest, layout[i].Y);
	}
	return highest;
}

float Building::GetHighX() {
	float highest = layout[0].X;
	for (int i = 0; i < layout.Num(); i++)
	{
		highest = fminf(highest, layout[i].X);
	}
	return highest;
}

float Building::GetPolygonArea()
{
	float a = 0.0f;
	float b = 0.0f;
	int mod = layout.Num();
	for (int i = 0; i < mod; i++) {
		a += layout[i].X * layout[(i + 1) % mod].Y;
		b += layout[i].Y * layout[(i + 1) % mod].X;
	}

	return fabsf((a - b) / 2);
}

FVector2D Building::GetPolygonCenter()
{
	int mod = layout.Num();
	float area = 6 * GetPolygonArea();
	float centerX = 0.0f;
	float centerY = 0.0f;
	for (int m = 0; m < mod; m++)
	{
		int secondIndex = (m + 1) % mod;
		centerX += (layout[m].X + layout[secondIndex].X) * (layout[m].X * layout[secondIndex].Y - layout[m].Y * layout[secondIndex].X);
		centerY += (layout[m].Y + layout[secondIndex].Y) * (layout[m].X * layout[secondIndex].Y - layout[m].Y * layout[secondIndex].X);
	}
	centerX /= area;
	centerY /= area;

	return FVector2D(centerX, centerY);
}

bool Building::CheckIntersection(FVector2D a, FVector2D b, FVector2D c, FVector2D d)
{
	// Find the four orientations
	int o1 = Orientation(a, b, c);
	int o2 = Orientation(a, b, d);
	int o3 = Orientation(c, d, a);
	int o4 = Orientation(c, d, b);

	// General case 
	if (o1 != o2 && o3 != o4)
		return true;

	// Special Cases 
	// p1, q1 and p2 are colinear and p2 lies on segment p1q1 
	if (o1 == 0 && OnSegment(a, c, b)) return true;

	// p1, q1 and q2 are colinear and q2 lies on segment p1q1 
	if (o2 == 0 && OnSegment(a, d, b)) return true;

	// p2, q2 and p1 are colinear and p1 lies on segment p2q2 
	if (o3 == 0 && OnSegment(c, a, d)) return true;

	// p2, q2 and q1 are colinear and q1 lies on segment p2q2 
	if (o4 == 0 && OnSegment(c, b, d)) return true;

	return false;
}
