// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "QuadTree.h"

UENUM(BlueprintType)
enum class EBuildingType : uint8
{
	BT_Residental/* 	UMETA(DisplayName = "ResidentalBuilding")*/,
	BT_Industrial,
	BT_Commercial,
	BT_Park
};

class PROCEDURALCITY_API Building
{
public:
	Building();
	Building(TArray<FVector2D> layout, float height, EBuildingType buildingType);
	~Building();
	Building Split(FVector2D firstSplitLocation, FVector2D secondSplitLocation, int firstSplitIndex, int secondSplitIndex, bool createNewBuilding, float minSize, FVector2D offset/*, QuadTree<Building> qTree*/);
	bool OnSegment(FVector2D p, FVector2D q, FVector2D r);
	int Orientation(FVector2D p, FVector2D q, FVector2D r);
	bool CheckForIntersection(Building other);
	float GetPolygonArea();
	FVector2D GetPolygonCenter();
	FORCEINLINE	bool operator==(const Building& V) const
	{
		return layout == V.layout && height == V.height && buildingType == V.buildingType;
	};

private:
	//lines are ab and cd
	bool CheckIntersection(FVector2D a, FVector2D b, FVector2D c, FVector2D d);

public:
	TArray<FVector2D> layout;
	float height;
	float GetLowY();
	float GetLowX();
	float GetHighY();
	float GetHighX();
	EBuildingType buildingType;
};
