// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/GameModeBase.h"
#include "ProceduralCityGameModeBase.generated.h"

/**
*
*/
UCLASS()
class PROCEDURALCITY_API AProceduralCityGameModeBase : public AGameModeBase
{
	GENERATED_BODY()
public:
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float highway_minLength;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float highway_maxLength;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float highway_width;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float sub_minLength;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float sub_maxLength;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float sub_width;
	//needs to be an odd number
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		int highway_sampleAmount;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		int sub_sampleAmount;
	//maximum angle end segment is allowed to vary from original direction without snapping (concerns sampling on globalGoals and adjustSegment)
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float highway_maxDeviationAngle;
	//maximum angle end segment is allowed to vary from original direction without snapping (concerns sampling on globalGoals and adjustSegment)
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float sub_maxDeviationAngle;
	//higher values increase potential for deviation of direction
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float highway_deviationWeight;
	//higher values increase potential for deviation of direction
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float sub_deviationWeight;
	//higher values increase potential to take smallest height difference
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float sub_heightWeight;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float highway_splitProbability;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float sub_splitProbability;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float highway_splitThreshold;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float sub_splitThreshold;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float minIntersectionAngle;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float minDistanceBetweenStartPoints;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float snapDistance;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float minBuildingArea;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float minBuildingLength;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float industrialHeightMin;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float industrialHeightMax;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float commercialHeightMin;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float commercialHeightMax;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float residentalHeightMin;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float residentalHeightMax;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float minGrowthValue;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		int seed;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		int segmentLimit;
	//TODO needs to be added to quadtree instead of local variable
	int quadTreeMaximumObjects;
	int quadTreeMaximumDepth;
	//TODO bools need to be implemented
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		bool onlyHighways;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		bool ignoreConstraints;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		bool startAtCenter;
};
