#pragma once

#include "CoreMinimal.h"
//#include "Segment.h"
#include "Bounds.h"

//template<class T>
//class Data {
//public:
//	TArray<T> objects;
//};

template<class T>
class PROCEDURALCITY_API QuadTree
{
public:	
	QuadTree() {
		depth = -1;
	};
	QuadTree(int pDepth, Bounds pBounds) {
		depth = pDepth;
		bounds = pBounds;
		nodes.Add(QuadTree());
		nodes.Add(QuadTree());
		nodes.Add(QuadTree());
		nodes.Add(QuadTree());
	};

protected:
	bool HasNodes() {
		return nodes[0].depth != -1;
	};
	void SplitTree() {
		float subWidth = bounds.width / 2;
		float subHeight = bounds.height / 2;
		float x = bounds.x;
		float y = bounds.y;

		nodes[0] = QuadTree(depth + 1, Bounds(x + subWidth, y, subWidth, subHeight));
		nodes[1] = QuadTree(depth + 1, Bounds(x, y, subWidth, subHeight));
		nodes[2] = QuadTree(depth + 1, Bounds(x, y + subHeight, subWidth, subHeight));
		nodes[3] = QuadTree(depth + 1, Bounds(x + subWidth, y + subHeight, subWidth, subHeight));
	};
	int GetTreeNode(T newSegment) {
		int index = -1;
		double verticalMidpoint = bounds.x + (bounds.width / 2);
		double horizontalMidpoint = bounds.y + (bounds.height / 2);

		//TODO check if x and y are correct or inversed
		bool topQuadrant = newSegment.GetHighY() < horizontalMidpoint;
		bool bottomQuadrant = newSegment.GetLowY() >= horizontalMidpoint;

		// left quadrants
		if (newSegment.GetHighX() < verticalMidpoint) {
			if (topQuadrant) {
				index = 1;
			}
			else if (bottomQuadrant) {
				index = 2;
			}
		}

		// right quadrants
		if (newSegment.GetLowX() >= verticalMidpoint) {
			if (topQuadrant) {
				index = 0;
			}
			else if (bottomQuadrant) {
				index = 3;
			}
		}

		return index;
	};

public:	
	void AddSegment(T newSegment) {
		if (HasNodes()) {
			int index = GetTreeNode(newSegment);
			if (index != -1) {
				nodes[index].AddSegment(newSegment);
				return;
			}
		}

		objects.AddUnique(newSegment);

		if (objects.Num() > maxElements && depth < maxDepth) {
			if (!HasNodes()) {
				SplitTree();
			}

			int i = 0;
			while (i < objects.Num()) {
				int index = GetTreeNode(objects[i]);
				if (index != -1) {
					nodes[index].AddSegment(objects[i]);
					objects.RemoveAt(i);
				}
				else {
					i++;
				}
			}
		}
	};
	void Clear() {
		objects.Empty();
		nodes.Empty();
		nodes.Add(QuadTree());
		nodes.Add(QuadTree());
		nodes.Add(QuadTree());
		nodes.Add(QuadTree());
	};
	TArray<QuadTree<T>> GetNodes() {
		return nodes;
	};
	Bounds GetBounds() {
		return bounds;
	};
	// returns objects from nodes inside current node
	TArray<T> GetObjectsInNode() {
		TArray<T> returnObjects;
		if (HasNodes()) {
			for (int i = 0; i < nodes.Num(); i++)
			{
				returnObjects.Append(nodes[i].GetObjectsInNode());
			}
		}

		returnObjects.Append(objects);

		return returnObjects;
	};
	// returns objects of the node where this segment is located in
	TArray<T> GetObjectsInNode(T newSegment) {
		int index = GetTreeNode(newSegment);
		TArray<T> returnObjects;
		if (HasNodes()) {
			// check if the newSegment fits into a subtree
			if (index != -1) returnObjects.Append(nodes[index].GetObjectsInNode(newSegment));
			else {
				// if newSegment is inside current node, get segments inside subtrees
				for (int i = 0; i < nodes.Num(); i++)
				{
					returnObjects.Append(nodes[i].GetObjectsInNode());
				}
			}
		}
		// return elements of higher nodes, to get the intersecting elements as well
		returnObjects.Append(objects);

		return returnObjects;
	};
	TArray<T> GetAllObjects() {
		TArray<T> returnObjects;
		returnObjects.Append(objects);
		for (int i = 0; i < nodes.Num(); i++)
		{
			returnObjects.Append(nodes[i].GetAllObjects());
		}
		return returnObjects;
	};

	int GetDepth() {
		return depth;
	}

private:
	int maxElements = 10;
	int maxDepth = 5;
	int depth;
	TArray</*Data<T>*/T> objects;
	TArray<QuadTree<T>> nodes;
	Bounds bounds;

};
