#pragma once

#include "CoreMinimal.h"
#include "QuadTree.h"
#include "Building.h"

class PROCEDURALCITY_API Segment
{
public:	
	Segment();
	Segment(FVector2D start, FVector2D end, bool isHighway);
	Segment(FVector2D start, FVector2D end, bool isHighway, int category);

public:
	void SetStart(FVector2D newStart);
	void SetEnd(FVector2D newEnd);
	FVector2D GetStart();
	FVector2D GetEnd();
	float length;
	float GetOriginalLength();
	bool isHighway;
	bool isEndConnected;
	FVector2D direction;
	FVector2D GetOriginalDirection();
	TArray<FVector2D> furtherConnections;
	int category;

	void Split(FVector2D splitLocation, QuadTree<Segment> qTree);
	bool OnSegment(FVector2D p, FVector2D q, FVector2D r);
	int Orientation(FVector2D p, FVector2D q, FVector2D r);
	bool CheckForIntersection(Segment segment);
	FVector2D GetIntersectionLocation(Segment segment);
	int CheckStreetEndInside(Building building);
	//returns new split Building and changes the old one
	Building CheckIntersectionBuilding(Building &building, bool createNewBuilding, float minSize, float width);
	float GetLowY();
	float GetLowX();
	float GetHighY();
	float GetHighX();
	FORCEINLINE	bool operator==(const Segment& V) const
	{
		return start == V.start && end == V.end;
	};

private:
	FVector2D start;
	FVector2D end;
	float originalLength;
	FVector2D originalDirection;
};
