// Fill out your copyright notice in the Description page of Project Settings.

#include "StreetGenerator.h"
#include "Misc/Paths.h"
#include "Misc/PackageName.h"
#include "TextureResource.h"
#include "AssetRegistryModule.h"
#include "BuildingActor.h"

// Sets default values
AStreetGenerator::AStreetGenerator()
{
	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

}

// Called when the game starts or when spawned
void AStreetGenerator::BeginPlay()
{
	Super::BeginPlay();
	config = Cast<AProceduralCityGameModeBase>(GetWorld()->GetAuthGameMode());
	pN = PerlinNoise(config->seed);
	pNHeight = PerlinNoise(config->seed + 1);
	qT_Street = QuadTree<Segment>(0, Bounds(0.0f, 0.0f, areaWidth, areaHeight));
	qT_Building = QuadTree<Building>(0, Bounds(0.0f, 0.0f, areaWidth, areaHeight));
	//ConstructCity();
}

// Called every frame
void AStreetGenerator::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);

}

TArray<Segment> AStreetGenerator::GlobalGoals(Segment segment) {
	TArray<Segment> newBranches;
	float newLength;
	if (segment.isHighway) newLength = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(config->highway_minLength, config->highway_maxLength);
	else newLength = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(config->sub_minLength, config->sub_maxLength);
	Segment newSegment = Segment(segment.GetEnd(), segment.GetEnd() + segment.direction * newLength, segment.isHighway, segment.category);

	float straightPop = GetNoiseValue(newSegment.GetEnd().X, newSegment.GetEnd().Y);
	if (segment.isHighway) {
		//TODO wrap functionality
		float maxPop = straightPop;
		Segment bestSegment = newSegment;
		for (int i = 0; i < config->highway_sampleAmount; i++) {
			//deviation of straigt segment
			float deviatedAngle = i * (2 * config->highway_maxDeviationAngle / (config->highway_sampleAmount - 1)) - config->highway_maxDeviationAngle;
			Segment curSegment = Segment(segment.GetEnd(), segment.GetEnd() + segment.direction.GetRotated(deviatedAngle) * newLength, segment.isHighway, segment.category);
			float curPop = GetNoiseValue(curSegment.GetEnd().X, curSegment.GetEnd().Y);
			if (curPop * FMath::Cos(deviatedAngle / config->highway_deviationWeight) > maxPop) {
				maxPop = curPop;
				bestSegment = curSegment;
			}
		}
		newBranches.Add(bestSegment);

		float rotAngle = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(-1.0f, 1.0f) * (90.0f - config->minIntersectionAngle);

		if (maxPop > config->highway_splitThreshold) {
			if (FRandomStream(bestSegment.GetEnd().X).FRand() < config->highway_splitProbability) {
				FVector2D newDirection = segment.direction.GetRotated(-90.0f + rotAngle);
				float newLengthSplit = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(config->highway_minLength, config->highway_maxLength);
				if (segment.category >= maxCategory) newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, false));
				else newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, segment.isHighway, segment.category + 1));
			}
			else if (FRandomStream(bestSegment.GetEnd().Y).FRand() < config->highway_splitProbability) {
				FVector2D newDirection = segment.direction.GetRotated(90.0f + rotAngle);
				float newLengthSplit = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(config->highway_minLength, config->highway_maxLength);
				if (segment.category >= maxCategory) newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, false));
				else newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, segment.isHighway, segment.category + 1));
			}
			else if (FRandomStream(bestSegment.GetEnd().X + bestSegment.GetEnd().Y).FRand() < config->highway_splitProbability) {
				//split in both ways
				FVector2D newDirection = segment.direction.GetRotated(90.0f + rotAngle);
				float newLengthSplit = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(config->highway_minLength, config->highway_maxLength);
				if (segment.category >= maxCategory) {
					newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, false));
					newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() - newDirection * newLengthSplit, false));
				}
				else {
					newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, !segment.isHighway));
					newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() - newDirection * newLengthSplit, !segment.isHighway));
				}
			}
		}
	}
	else {
		float maxPop = straightPop;
		Segment bestSegment = newSegment;
		float startHeight = GetHeight(segment.GetEnd().X, segment.GetEnd().Y);
		for (int i = 0; i < config->sub_sampleAmount; i++) {
			float deviatedAngle = i * (2 * config->sub_maxDeviationAngle / (config->sub_sampleAmount - 1)) - config->sub_maxDeviationAngle;
			Segment curSegment = Segment(segment.GetEnd(), segment.GetEnd() + segment.direction.GetRotated(deviatedAngle) * newLength, segment.isHighway);
			float curPop = GetNoiseValue(curSegment.GetEnd().X, curSegment.GetEnd().Y);
			float heightDiv = fabsf(GetHeight(curSegment.GetEnd().X, curSegment.GetEnd().Y) - startHeight);
			if (curPop * FMath::Cos(deviatedAngle / config->sub_deviationWeight) / (heightDiv * config->sub_heightWeight + 1) > maxPop) {
				maxPop = curPop;
				bestSegment = curSegment;
			}
		}
		newBranches.Add(bestSegment);

		float rotAngle = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(-1.0f, 1.0f) * (90.0f - config->minIntersectionAngle);

		if (maxPop > config->sub_splitThreshold) {
			if (FRandomStream(bestSegment.GetEnd().X).FRand() < config->sub_splitProbability) {
				FVector2D newDirection = segment.direction.GetRotated(-90.0f + rotAngle);
				float newLengthSplit = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(config->sub_minLength, config->sub_maxLength);
				newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, segment.isHighway));
			}
			else if (FRandomStream(bestSegment.GetEnd().Y).FRand() < config->sub_splitProbability) {
				FVector2D newDirection = segment.direction.GetRotated(90.0f + rotAngle);
				float newLengthSplit = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(config->sub_minLength, config->sub_maxLength);
				newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, segment.isHighway));
			}
			else if (FRandomStream(bestSegment.GetEnd().X + bestSegment.GetEnd().Y).FRand() < config->sub_splitProbability / 2.0f) {
				//split in both ways
				FVector2D newDirection = segment.direction.GetRotated(90.0f + rotAngle);
				float newLengthSplit = FRandomStream(segment.GetEnd().X + segment.GetEnd().Y).FRandRange(config->sub_minLength, config->sub_maxLength);
				newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() + newDirection * newLengthSplit, segment.isHighway));
				newBranches.Add(Segment(segment.GetEnd(), segment.GetEnd() - newDirection * newLengthSplit, segment.isHighway));
			}
		}
	}

	return newBranches;
}

bool AStreetGenerator::LocalConstraints(Segment &segment) {
	if (config->ignoreConstraints) return true;
	//TODO check for error to create segment with length == 0.0f, currently fixed with if(s.length > 0.0f) localSegments.AddUnique(s); in Generate()
	if (segment.GetStart().X < 0.0f || segment.GetStart().Y < 0.0f || segment.GetStart().X > areaWidth || segment.GetStart().Y > areaHeight) return false;
	//check if segment need adjustments or cant be adjusted to fit at all
	if (!AdjustSegment(segment, true)) return false;

	FVector2D tempEnd = segment.GetEnd() + segment.direction * config->snapDistance;
	Segment temp = Segment(segment.GetEnd(), tempEnd, segment.isHighway);
	TArray<Segment> newSegments;
	TArray<Segment> allStreets = qT_Street.GetAllObjects();
	TArray<Segment> other = qT_Street.GetObjectsInNode(Segment(segment.GetStart(), tempEnd, segment.isHighway));
	for (int i = 0; i < other.Num(); i++) {
		allStreets.Remove(other[i]);
		if (!segment.isEndConnected) {
			// snap to crossing within radius check
			if ((segment.GetEnd() - other[i].GetEnd()).Size() <= config->snapDistance && segment.GetEnd() != other[i].GetEnd()) {
				segment.GetEnd() = other[i].GetEnd();
				segment.furtherConnections.Add(other[i].GetEnd() - segment.GetStart());
				segment.isEndConnected = true;
			}
			else if ((segment.GetEnd() - other[i].GetStart()).Size() <= config->snapDistance && segment.GetEnd() != other[i].GetStart()) {
				segment.GetEnd() = other[i].GetStart();
				segment.furtherConnections.Add(other[i].GetStart() - segment.GetStart());
				segment.isEndConnected = true;
			}

			//  extend road and create new intersection within snap distance
			else if (temp.CheckForIntersection(other[i])) {
				if (temp.GetStart() == other[i].GetStart() || temp.GetStart() == other[i].GetEnd()) continue;
				else {
					float midAngle = FMath::Acos(FVector2D::DotProduct(segment.direction, other[i].direction)) * 180 / PI;
					if (midAngle > 90) midAngle = 180 - midAngle;
					if (midAngle < config->minIntersectionAngle) {
						if (segment.furtherConnections.Num() > 0) continue;
						else return false;
					}
				}
				FVector2D intersection = temp.GetIntersectionLocation(other[i]);
				if (isinf(intersection.X) || isnan(intersection.X)) {
					int m3 = 4;
					FVector2D intersection2 = temp.GetIntersectionLocation(other[i]);
				}
				// might be redundant because theese points should have been checked with crossing snapping
				if (!(other[i].GetStart() == intersection || other[i].GetEnd() == intersection)) newSegments.Add(SplitSegment(other[i], intersection));
				segment.GetEnd() = intersection;
				segment.furtherConnections.AddUnique(intersection - segment.GetStart());
				segment.isEndConnected = true;
			}
		}

		// intersection check
		if (segment.CheckForIntersection(other[i])) {
			// dont check between origin and its new segments
			if (segment.GetStart() == other[i].GetEnd() || segment.GetEnd() == other[i].GetStart() || segment.GetEnd() == other[i].GetEnd()) continue;
			// check angle if both segments have same origin
			if (segment.GetStart() == other[i].GetStart()) {
				float midAngle = FMath::Acos(FVector2D::DotProduct(segment.direction, other[i].direction)) * 180 / PI;
				if (midAngle > 90) midAngle = 180 - midAngle;
				if (midAngle < config->minIntersectionAngle) {
					if (segment.furtherConnections.Num() > 0) continue;
					else return false;
				}
				else continue;
			}

			FVector2D intersection = segment.GetIntersectionLocation(other[i]);
			if (isinf(intersection.X) || isnan(intersection.X)) {
				int m3 = 4;
				FVector2D intersection2 = segment.GetIntersectionLocation(other[i]);
			}
			if (!(other[i].GetStart() == intersection || other[i].GetEnd() == intersection)) newSegments.Add(SplitSegment(other[i], intersection));
			if (segment.isEndConnected) {
				segment.furtherConnections.Insert(intersection - segment.GetStart(), segment.furtherConnections.Num() - 1);
			}
			else segment.furtherConnections.AddUnique(intersection - segment.GetStart());
			//to prevent snapping to the same element
			continue;
		}

	}
	for (int s = 0; s < other.Num(); s++)
	{
		allStreets.AddUnique(other[s]);
	}
	allStreets.Append(newSegments);
	qT_Street.Clear();
	for (int a = 0; a < allStreets.Num(); a++)
	{
		qT_Street.AddSegment(allStreets[a]);
	}
	return true;
}

void AStreetGenerator::Generate() {
	if (config->segmentLimit < qT_Street.GetAllObjects().Num()) return;

	TArray<Segment> newActivSegments;
	for (int i = 0; i < activSegments.Num(); i++)
	{
		TArray<Segment> newBranches = GlobalGoals(activSegments[i]);
		for (int j = 0; j < newBranches.Num(); j++)
		{
			bool valid = LocalConstraints(newBranches[j]);
			if (valid) {
				TArray<Segment> localSegments;
				newBranches[j].furtherConnections.Sort([](const FVector2D& A, const FVector2D& B) {
					return A.Size() < B.Size();
				});
				for (int b = 0; b < newBranches[j].furtherConnections.Num(); b++)
				{
					if (b == 0) {
						Segment s = Segment(newBranches[j].GetStart(), newBranches[j].GetStart() + newBranches[j].furtherConnections[b], newBranches[j].isHighway, newBranches[j].category);
						if (s.length > 0.0f) localSegments.AddUnique(s);
					}
					else {
						Segment s = Segment(newBranches[j].GetStart() + newBranches[j].furtherConnections[b - 1], newBranches[j].GetStart() + newBranches[j].furtherConnections[b], newBranches[j].isHighway, newBranches[j].category);
						if (s.length > 0.0f) localSegments.AddUnique(s);
					}
				}
				if (localSegments.Num() > 0) {
					for (int l = 0; l < localSegments.Num(); l++)
					{
						if (config->segmentLimit < qT_Street.GetAllObjects().Num()) break;
						//check if new segment intersects with existing buildings and update them
						FVector2D rotDirection = localSegments[l].direction.GetRotated(-90.0f) * config->minBuildingLength;
						FVector2D mid = localSegments[l].GetEnd() - localSegments[l].GetStart();
						float height = GetNoiseValue(mid.X, mid.Y);
						UpdateBuildings(localSegments[l], rotDirection);
						qT_Street.AddSegment(localSegments[l]);
						ConstructBuildings(localSegments[l], rotDirection, height);
					}
				}
				else {
					if (config->segmentLimit < qT_Street.GetAllObjects().Num()) break;

					if (!newBranches[j].isEndConnected) newActivSegments.Add(newBranches[j]);
					FVector2D rotDirection = newBranches[j].direction.GetRotated(-90.0f) * config->minBuildingLength;
					FVector2D mid = newBranches[j].GetEnd() - newBranches[j].GetStart();
					float height = GetNoiseValue(mid.X, mid.Y);
					UpdateBuildings(newBranches[j], rotDirection);
					qT_Street.AddSegment(newBranches[j]);
					ConstructBuildings(newBranches[j], rotDirection, height);
				}
			}
		}
	}
	activSegments.Empty();
	if (!(config->segmentLimit < qT_Street.GetAllObjects().Num())) activSegments = newActivSegments;
}

void AStreetGenerator::CreateInitialSetup() {
	TArray<FVector2D> startPoints;
	if (config->startAtCenter) startPoints.Add(FVector2D(areaWidth / 2.0f, areaHeight / 2.0f));
	else startPoints = GetStartpoints();
	if (startPoints.Num() == 0) startPoints.Add(FVector2D(683, 354));
	for (int i = 0; i < startPoints.Num(); i++)
	{
		TArray<Segment> newSegments;
		TArray<FVector2D> relativPoints;
		for (int p = 0; p < startPoints.Num(); p++)
		{
			relativPoints.Add(startPoints[p] - startPoints[i]);
		}
		relativPoints.Sort([](const FVector2D& A, const FVector2D& B) {
			return A.Size() < B.Size();
		});
		int highways = (FMath::Cos((startPoints[i].X / areaWidth + startPoints[i].Y / areaHeight) / 2 * 360) + 1) * 3;
		int subs = (FMath::Sin((startPoints[i].X / areaWidth + startPoints[i].Y / areaHeight) / 2 * 360) + 1) * 5;
		FVector2D straightH = FVector2D(0, config->highway_minLength);
		FVector2D straightS = FVector2D(0, config->sub_minLength);
		TArray<bool> connectedPoints;
		connectedPoints.Init(false, startPoints.Num());
		connectedPoints[0] = true;
		for (int h = 0; h < highways; h++)
		{
			int cV = 0;
			FVector2D start = startPoints[i];
			for (int cp = 0; cp < connectedPoints.Num(); cp++)
			{
				if (connectedPoints[cp] == false) {
					connectedPoints[cp] = true;
					FVector2D dir = relativPoints[cp];
					dir.Normalize();
					FVector2D end = start + dir * config->highway_minLength;
					Segment newSegment = Segment(start, end, true);
					newSegments.Add(newSegment);
					qT_Street.AddSegment(newSegment);
					break;
				}
				else cV++;
			}
			if (cV == connectedPoints.Num()) {
				float angle = FMath::Acos(startPoints[i].X / areaWidth) * 180 / PI + float(h) / float(highways) * 360.0f; //angle in degree
				FVector2D end = start + straightH.GetRotated(angle);
				Segment newSegment = Segment(start, end, true);
				newSegments.Add(newSegment);
				qT_Street.AddSegment(newSegment);
			}
		}
		for (int s = 0; s < subs; s++)
		{
			float angle = FMath::Acos(startPoints[i].X / areaWidth) * 180 / PI + float(s) / float(subs) * 360.0f; //angle in degree
			FVector2D start = startPoints[i];
			FVector2D end = start + straightS.GetRotated(angle);
			Segment newSegment = Segment(start, end, false);
			float valid = true;
			for (int l = 0; l < newSegments.Num(); l++)
			{
				float midAngle = FMath::Acos(FVector2D::DotProduct(newSegment.direction, newSegments[l].direction) / (newSegment.direction.Size() * newSegments[l].direction.Size())) * 180 / PI;
				if (midAngle > 90) midAngle = 180 - midAngle;
				if (midAngle < config->minIntersectionAngle) {
					valid = false;
					break;
				}
			}
			if (valid) {
				newSegments.Add(newSegment);
				qT_Street.AddSegment(newSegment);
			}
		}
		activSegments.Append(newSegments);
	}
}

TArray<FStreetSegment> AStreetGenerator::GetSegments()
{
	TArray<Segment> objs = qT_Street.GetAllObjects();
	TArray<FStreetSegment> returnValues;
	for (int i = 0; i < objs.Num(); i++)
	{
		returnValues.Add(FStreetSegment(objs[i].GetStart(), objs[i].GetEnd(), objs[i].isHighway));
	}
	return returnValues;
}

TArray<FBuildingSegment> AStreetGenerator::GetBuildings()
{
	TArray<Building> objs = qT_Building.GetAllObjects();
	TArray<FBuildingSegment> returnValues;
	for (int i = 0; i < objs.Num(); i++)
	{
		returnValues.Add(FBuildingSegment(objs[i].layout, objs[i].height, objs[i].buildingType));
	}
	return returnValues;
}

TArray<FStreetSegment> AStreetGenerator::GetActivSegments()
{
	TArray<FStreetSegment> returnValues;
	for (int i = 0; i < activSegments.Num(); i++)
	{
		returnValues.Add(FStreetSegment(activSegments[i].GetStart(), activSegments[i].GetEnd(), activSegments[i].isHighway));
	}
	return returnValues;
}

void AStreetGenerator::MakeNoiseP()
{
	float maxValue = 0.0f;
	float maxValueHeight = 0.0f;
	uint8* pixels = new uint8[areaWidth * areaHeight * 4];
	for (int x = 0; x < areaWidth; x++)
	{
		for (int y = 0; y < areaHeight; y++)
		{
			float newValue;
			float newValueHeight;
			if (usePersistance) {
				newValue = pN.OctaveNoise((double)x / areaWidth * shiftX, (double)y / areaHeight * shiftY, octaves, persistence);
				newValueHeight = pNHeight.OctaveNoise((double)x / areaWidth * shiftX, (double)y / areaHeight * shiftY, octaves, persistence);
			}
			else {
				newValue = pN.OctaveNoise((double)x / areaWidth * shiftX, (double)y / areaHeight * shiftY, octaves, frequenceChange, amplitudeChange);
				newValueHeight = pNHeight.OctaveNoise((double)x / areaWidth * shiftX, (double)y / areaHeight * shiftY, octaves, frequenceChange, amplitudeChange);
			}
			noise.Add(newValue);
			height.Add(newValueHeight);
			if (newValue > maxValue) maxValue = newValue;
			if (newValueHeight > maxValueHeight) maxValueHeight = newValueHeight;
		}
	}
	//normalizes the values
	for (int i = 0; i < noise.Num(); i++)
	{
		if (normalizeNoise) {
			noise[i] /= maxValue;
			height[i] /= maxValueHeight;
		}
		uint8 curColor = noise[i] * 255;
		pixels[4 * i] = curColor;
		pixels[4 * i + 1] = curColor;
		pixels[4 * i + 2] = curColor;
		pixels[4 * i + 3] = 255;
	}

	//if (createNoiseTexture) {
	//	// Create Package
	//	FString fileName = FString("MyTexture");
	//	FString pathPackage = FString("/Game/MyTextures/");
	//	pathPackage += fileName;
	//	//FString absolutePathPackage = FPaths::ProjectContentDir() + "/MyTextures/";

	//	//FPackageName::RegisterMountPoint(*pathPackage, *absolutePathPackage);

	//	UPackage * Package = CreatePackage(nullptr, *pathPackage);
	//	Package->FullyLoad();

	//	// Create the Texture
	//	//FName textureName = MakeUniqueObjectName(Package, UTexture2D::StaticClass(), FName(*fileName));
	//	UTexture2D* texture = NewObject<UTexture2D>(Package, *fileName, RF_Public | RF_Standalone | RF_MarkAsRootSet);

	//	// Texture Settings
	//	texture->AddToRoot();
	//	texture->PlatformData = new FTexturePlatformData();
	//	texture->PlatformData->SizeX = areaWidth;
	//	texture->PlatformData->SizeY = areaHeight;
	//	texture->PlatformData->PixelFormat = PF_R8G8B8A8;

	//	// Passing the pixels information to the texture
	//	FTexture2DMipMap* Mip = new(texture->PlatformData->Mips) FTexture2DMipMap();
	//	Mip->SizeX = areaWidth;
	//	Mip->SizeY = areaHeight;
	//	Mip->BulkData.Lock(LOCK_READ_WRITE);
	//	uint8* TextureData = (uint8 *)Mip->BulkData.Realloc(areaHeight * areaWidth * sizeof(uint8) * 4);
	//	FMemory::Memcpy(TextureData, pixels, sizeof(uint8) * areaHeight * areaWidth * 4);
	//	Mip->BulkData.Unlock();

	//	// Updating Texture & mark it as unsaved
	//	texture->Source.Init(areaWidth, areaHeight, 1, 1, ETextureSourceFormat::TSF_RGBA8, pixels);
	//	texture->UpdateResource();
	//	Package->MarkPackageDirty();

	//	FAssetRegistryModule::AssetCreated(texture);
	//	FString PackageFileName = FPackageName::LongPackageNameToFilename(pathPackage, FPackageName::GetAssetPackageExtension());
	//	bool bSaved = UPackage::SavePackage(Package, texture, EObjectFlags::RF_Public | EObjectFlags::RF_Standalone, *PackageFileName, GError, nullptr, true, true, SAVE_NoError);

	//	UE_LOG(LogTemp, Warning, TEXT("Texture created: %s, max: %f"), *fileName, maxValue);

	//	delete[] pixels;
	//}
}

TArray<FVector2D> AStreetGenerator::GetPoints()
{
	TArray<Segment> objs = qT_Street.GetAllObjects();
	TArray<FVector2D> returnValues;
	for (int i = 0; i < objs.Num(); i++)
	{
		returnValues.AddUnique(objs[i].GetStart());
		returnValues.AddUnique(objs[i].GetEnd());
	}
	return returnValues;
}

TArray<FBuildingSegment> AStreetGenerator::DrawQuadtreeBP()
{
	return DrawQuadtree(qT_Building);
}

void AStreetGenerator::VisualizeBuildings()
{
	TArray<Building> buildings = qT_Building.GetAllObjects();

	for (int b = 0; b < buildings.Num(); b++)
	{
		TArray<int32> tri;
		TArray<FVector> vertices;
		TArray<FVector> normals;
		TArray<FVector2D> uV0s;
		TArray<FLinearColor> vertexColors;
		UMaterial* material = materials.Last();
		FVector2D center = buildings[b].GetPolygonCenter();
		FVector center3D = FVector(center.X, center.Y, 0.0f);
		int mod = buildings[b].layout.Num();

		//setup triangles
		switch (mod)
		{
		case 3:
			tri.Add(0);
			tri.Add(2);
			tri.Add(1);
			tri.Add(0);
			tri.Add(5);
			tri.Add(3);
			tri.Add(0);
			tri.Add(2);
			tri.Add(5);
			tri.Add(1);
			tri.Add(3);
			tri.Add(4);
			tri.Add(1);
			tri.Add(0);
			tri.Add(3);
			tri.Add(2);
			tri.Add(4);
			tri.Add(5);
			tri.Add(2);
			tri.Add(1);
			tri.Add(4);
			break;
		case 4:
			tri.Add(0);
			tri.Add(2);
			tri.Add(1);
			tri.Add(0);
			tri.Add(3);
			tri.Add(2);
			tri.Add(0);
			tri.Add(7);
			tri.Add(4);
			tri.Add(0);
			tri.Add(3);
			tri.Add(7);
			tri.Add(1);
			tri.Add(4);
			tri.Add(5);
			tri.Add(1);
			tri.Add(0);
			tri.Add(4);
			tri.Add(2);
			tri.Add(5);
			tri.Add(6);
			tri.Add(2);
			tri.Add(1);
			tri.Add(5);
			tri.Add(3);
			tri.Add(6);
			tri.Add(7);
			tri.Add(3);
			tri.Add(2);
			tri.Add(6);
			break;
		case 5:
			tri.Add(0);
			tri.Add(2);
			tri.Add(1);
			tri.Add(0);
			tri.Add(3);
			tri.Add(2);
			tri.Add(0);
			tri.Add(4);
			tri.Add(3);
			tri.Add(0);
			tri.Add(9);
			tri.Add(5);
			tri.Add(0);
			tri.Add(4);
			tri.Add(9);
			tri.Add(1);
			tri.Add(5);
			tri.Add(6);
			tri.Add(1);
			tri.Add(0);
			tri.Add(5);
			tri.Add(2);
			tri.Add(6);
			tri.Add(7);
			tri.Add(2);
			tri.Add(1);
			tri.Add(6);
			tri.Add(3);
			tri.Add(7);
			tri.Add(8);
			tri.Add(3);
			tri.Add(2);
			tri.Add(7);
			tri.Add(4);
			tri.Add(8);
			tri.Add(9);
			tri.Add(4);
			tri.Add(3);
			tri.Add(8);
			break;
		case 6:
			tri.Add(0);
			tri.Add(2);
			tri.Add(1);
			tri.Add(0);
			tri.Add(3);
			tri.Add(2);
			tri.Add(0);
			tri.Add(5);
			tri.Add(3);
			tri.Add(0);
			tri.Add(11);
			tri.Add(6);
			tri.Add(0);
			tri.Add(5);
			tri.Add(11);
			tri.Add(1);
			tri.Add(6);
			tri.Add(7);
			tri.Add(1);
			tri.Add(0);
			tri.Add(6);
			tri.Add(2);
			tri.Add(7);
			tri.Add(8);
			tri.Add(2);
			tri.Add(1);
			tri.Add(7);
			tri.Add(3);
			tri.Add(8);
			tri.Add(9);
			tri.Add(3);
			tri.Add(2);
			tri.Add(8);
			tri.Add(4);
			tri.Add(9);
			tri.Add(10);
			tri.Add(4);
			tri.Add(3);
			tri.Add(9);
			tri.Add(5);
			tri.Add(10);
			tri.Add(11);
			tri.Add(5);
			tri.Add(4);
			tri.Add(10);
			tri.Add(5);
			tri.Add(4);
			tri.Add(3);
			tri.Add(6);
			tri.Add(8);
			tri.Add(7);
			tri.Add(6);
			tri.Add(9);
			tri.Add(8);
			tri.Add(6);
			tri.Add(11);
			tri.Add(9);
			tri.Add(11);
			tri.Add(10);
			tri.Add(9);
			break;
		default:
			break;
		}
		//setup vertices
		for (int i = 0; i < 2 * mod; i++)
		{
			int h = i / mod;
			FVector newVec = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, h * buildings[b].height);
			vertices.Add(newVec);
			//TODO search solution to setup correct normals and uv0s
			FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, h * buildings[b].height);
			FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, h * buildings[b].height);
			FVector v3;
			if (h == 0) v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
			//else v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, 0.0f);
			FVector normal = GetNormal(v1 - newVec, v3 - newVec) + GetNormal(v3 - newVec, v2 - newVec);
			if (h == 0) normal = (normal - FVector().UpVector) / 3;
			//else normal = (normal + FVector().UpVector) / 3;
			normals.Add(normal);
			uV0s.Add(FVector2D(0.0f, 0.0f));
			switch (buildings[b].buildingType)
			{
			case EBuildingType::BT_Commercial:
				vertexColors.Add(FLinearColor(1.0f, 0.5f, 0.0f));
				material = materials[0];
				break;
			case EBuildingType::BT_Industrial:
				vertexColors.Add(FLinearColor(1.0f, 0.0f, 0.0f));
				material = materials[1];
				break;
			case EBuildingType::BT_Park:
				vertexColors.Add(FLinearColor(0.0f, 1.0f, 0.0f));
				material = materials[2];
				break;
			case EBuildingType::BT_Residental:
				vertexColors.Add(FLinearColor(0.0f, 0.0f, 1.0f));
				material = materials[3];
				break;
			default:
				material = materials.Last();
				break;
			}
		}
		
		//possible roof variation
		float random = FRandomStream(buildings[b].layout[0].X + buildings[b].layout[0].Y).FRand();
		switch (mod)
		{
		case 3:
			if (random < 0.17f) {
				//first point moved up
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec = FVector(buildings[b].layout[0].X - center.X, buildings[b].layout[0].Y - center.Y, buildings[b].height + 10);
				vertices.Add(addVec);
				tri.Add(3);
				tri.Add(6);
				tri.Add(4);
				tri.Add(4);
				tri.Add(6);
				tri.Add(5);
				tri.Add(5);
				tri.Add(6);
				tri.Add(3);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, 0.0f);
					FVector normal = GetNormal(v3 - v0, v1 - v0) + GetNormal(v2 - v0, v3 - v0);
					if (i == 0) normal = normal / 2;
					else if (i == 1) normal = (normal + GetNormal(v1 - v0, addVec - v0)) / 3;
					else normal = (normal + GetNormal(addVec - v0, v2 - v0)) / 3;
					normals.Add(normal);
				}
				FVector v0 = FVector(buildings[b].layout[1].X - center.X, buildings[b].layout[1].Y - center.Y, buildings[b].height);
				FVector v1 = FVector(buildings[b].layout[2].X - center.X, buildings[b].layout[2].Y - center.Y, buildings[b].height);
				FVector v2 = -FVector().UpVector;
				FVector normal = (GetNormal(v0 - addVec, v1 - addVec) + GetNormal(v2, v0 - addVec) + GetNormal(v1 - addVec, v2)) / 3;
				normals.Add(normal);
			}
			else if (random < 0.34f) {
				//second point moved up
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec = FVector(buildings[b].layout[1].X - center.X, buildings[b].layout[1].Y - center.Y, buildings[b].height + 10);
				vertices.Add(addVec);
				tri.Add(3);
				tri.Add(6);
				tri.Add(4);
				tri.Add(4);
				tri.Add(6);
				tri.Add(5);
				tri.Add(5);
				tri.Add(6);
				tri.Add(3);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, 0.0f);
					FVector normal = GetNormal(v3 - v0, v1 - v0) + GetNormal(v2 - v0, v3 - v0);
					if (i == 0) normal = (normal + GetNormal(addVec - v0, v2 - v0)) / 3;
					else if (i == 1) normal = normal / 2;
					else normal = (normal + GetNormal(v1 - v0, addVec - v0)) / 3;
					normals.Add(normal);
				}
				FVector v0 = FVector(buildings[b].layout[2].X - center.X, buildings[b].layout[2].Y - center.Y, buildings[b].height);
				FVector v1 = FVector(buildings[b].layout[0].X - center.X, buildings[b].layout[0].Y - center.Y, buildings[b].height);
				FVector v2 = -FVector().UpVector;
				FVector normal = (GetNormal(v0 - addVec, v1 - addVec) + GetNormal(v2, v0 - addVec) + GetNormal(v1 - addVec, v2)) / 3;
				normals.Add(normal);
			}
			else if (random < 0.5f) {
				//third point moved up
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec = FVector(buildings[b].layout[2].X - center.X, buildings[b].layout[2].Y - center.Y, buildings[b].height + 10);
				vertices.Add(addVec);
				tri.Add(3);
				tri.Add(6);
				tri.Add(4);
				tri.Add(4);
				tri.Add(6);
				tri.Add(5);
				tri.Add(5);
				tri.Add(6);
				tri.Add(3);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, 0.0f);
					FVector normal = GetNormal(v3 - v0, v1 - v0) + GetNormal(v2 - v0, v3 - v0);
					if (i == 0)normal = (normal + GetNormal(v1 - v0, addVec - v0)) / 3;
					else if (i == 1) (normal + GetNormal(addVec - v0, v2 - addVec)) / 3;
					else normal = normal / 2;
					normals.Add(normal);
				}
				FVector v0 = FVector(buildings[b].layout[0].X - center.X, buildings[b].layout[0].Y - center.Y, buildings[b].height);
				FVector v1 = FVector(buildings[b].layout[1].X - center.X, buildings[b].layout[1].Y - center.Y, buildings[b].height);
				FVector v2 = -FVector().UpVector;
				FVector normal = (GetNormal(v0 - addVec, v1 - addVec) + GetNormal(v2, v0 - addVec) + GetNormal(v1 - addVec, v2)) / 3;
				normals.Add(normal);
			}
			else {
				//flat roof
				tri.Add(3);
				tri.Add(5);
				tri.Add(4);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, 0.0f);
					FVector normal = (GetNormal(v3 - v0, v1 - v0) + GetNormal(v2 - v0, v3 - v0) + FVector().UpVector) / 3;
					normals.Add(normal);
				}
			}
			break;
		case 4:
			if (random < 0.08f) {
				//gable roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec1 = FVector((buildings[b].layout[0].X + buildings[b].layout[1].X) / 2 - center.X, (buildings[b].layout[0].Y + buildings[b].layout[1].Y) / 2 - center.Y, buildings[b].height + 10);
				FVector addVec2 = FVector((buildings[b].layout[2].X + buildings[b].layout[3].X) / 2 - center.X, (buildings[b].layout[2].Y + buildings[b].layout[3].Y) / 2 - center.Y, buildings[b].height + 10);
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				tri.Add(4);
				tri.Add(8);
				tri.Add(5);
				tri.Add(5);
				tri.Add(8);
				tri.Add(6);
				tri.Add(6);
				tri.Add(8);
				tri.Add(9);
				tri.Add(6);
				tri.Add(9);
				tri.Add(7);
				tri.Add(7);
				tri.Add(9);
				tri.Add(8);
				tri.Add(7);
				tri.Add(8);
				tri.Add(4);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal = GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3);
					if (i == 0) normal = (normal + GetNormal(addVec1 - v0, v2 - v0)) / 3;
					else if (i == 1) normal = (normal + GetNormal(v1 - v0, addVec1 - v0)) / 3;
					else if (i == 2) normal = (normal + GetNormal(addVec2 - v0, v2 - v0)) / 3;
					else normal = (normal + GetNormal(v1 - v0, addVec2 - v0)) / 3;
					normals.Add(normal);
				}
				FVector v0 = FVector(buildings[b].layout[0].X - center.X, buildings[b].layout[0].Y - center.Y, buildings[b].height);
				FVector v1 = FVector(buildings[b].layout[1].X - center.X, buildings[b].layout[1].Y - center.Y, buildings[b].height);
				FVector v2 = FVector(buildings[b].layout[2].X - center.X, buildings[b].layout[2].Y - center.Y, buildings[b].height);
				FVector v3 = FVector(buildings[b].layout[3].X - center.X, buildings[b].layout[3].Y - center.Y, buildings[b].height);
				FVector normal1 = (GetNormal(addVec2 - addVec1, v0 - addVec1) + GetNormal(v1 - addVec1, addVec2 - addVec1) + GetNormal(v0 - addVec1, v1 - addVec1)) / 3;
				FVector normal2 = (GetNormal(addVec1 - addVec2, v2 - addVec2) + GetNormal(v3 - addVec2, addVec1 - addVec2) + GetNormal(v2 -addVec2, v3 - addVec2)) / 3;
				normals.Add(normal1);
				normals.Add(normal2);
			}
			else if (random < 0.16f) {
				//shed roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec1 = FVector(buildings[b].layout[1].X - center.X, buildings[b].layout[1].Y - center.Y, buildings[b].height + 10);
				FVector addVec2 = FVector(buildings[b].layout[2].X - center.X, buildings[b].layout[2].Y - center.Y, buildings[b].height + 10);
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				tri.Add(4);
				tri.Add(8);
				tri.Add(5);
				tri.Add(5);
				tri.Add(8);
				tri.Add(6);
				tri.Add(6);
				tri.Add(8);
				tri.Add(9);
				tri.Add(6);
				tri.Add(9);
				tri.Add(7);
				tri.Add(7);
				tri.Add(9);
				tri.Add(8);
				tri.Add(7);
				tri.Add(8);
				tri.Add(4);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal = GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3);
					if (i == 0) normal = (normal + GetNormal(addVec1 - v0, v2 - v0)) / 3;
					else if (i == 1 || i == 2) normal /= 2;
					else normal = (normal + GetNormal(v1 - v0, addVec2 - v0)) / 3;
					normals.Add(normal);
				}
				FVector v0 = FVector(buildings[b].layout[0].X - center.X, buildings[b].layout[0].Y - center.Y, buildings[b].height);
				FVector v1 = FVector(buildings[b].layout[3].X - center.X, buildings[b].layout[3].Y - center.Y, buildings[b].height);
				FVector v2 = -FVector().UpVector;
				FVector normal1 = (GetNormal(addVec2 - addVec1, v0 - addVec1) + GetNormal(v0 - addVec1, v2) + GetNormal(v2, addVec2 - addVec1)) / 3;
				FVector normal2 = (GetNormal(addVec1 - addVec2, v2 - addVec2) + GetNormal(v1 - addVec2, addVec1 - addVec2) + GetNormal(v2, v1 - addVec2)) / 3;
				normals.Add(normal1);
				normals.Add(normal2);
			}
			else if (random < 0.24f) {
				//pyramid roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec = FVector(/*center.X*/0.0f, /*center.Y*/0.0f, buildings[b].height + 10);
				vertices.Add(addVec);
				tri.Add(4);
				tri.Add(8);
				tri.Add(5);
				tri.Add(5);
				tri.Add(8);
				tri.Add(6);
				tri.Add(6);
				tri.Add(8);
				tri.Add(7);
				tri.Add(7);
				tri.Add(8);
				tri.Add(4);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal = (GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3) + GetNormal(addVec - v0, v2 - v0) + GetNormal(v1 - v0, addVec - v0)) / 4;
					normals.Add(normal);
				}
				normals.Add(FVector().UpVector);
			}
			else if (random < 0.32f) {
				//hipped roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());*/
				FVector localDir = FVector((buildings[b].layout[2] - buildings[b].layout[1]).X, (buildings[b].layout[2] - buildings[b].layout[1]).Y, 0.0f);
				float size = localDir.Size() * 0.2f;
				localDir.Normalize();
				FVector addVec1 = FVector((buildings[b].layout[0].X + buildings[b].layout[1].X) / 2 - center.X, (buildings[b].layout[0].Y + buildings[b].layout[1].Y) / 2 - center.Y, buildings[b].height + 10) + localDir * size;
				FVector addVec2 = FVector((buildings[b].layout[2].X + buildings[b].layout[3].X) / 2 - center.X, (buildings[b].layout[2].Y + buildings[b].layout[3].Y) / 2 - center.Y, buildings[b].height + 10) - localDir * size;
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				tri.Add(4);
				tri.Add(8);
				tri.Add(5);
				tri.Add(5);
				tri.Add(8);
				tri.Add(6);
				tri.Add(6);
				tri.Add(8);
				tri.Add(9);
				tri.Add(6);
				tri.Add(9);
				tri.Add(7);
				tri.Add(7);
				tri.Add(9);
				tri.Add(8);
				tri.Add(7);
				tri.Add(8);
				tri.Add(4);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal = GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3);
					if (i == 0) normal = (normal + GetNormal(addVec1 - v0, v2 - v0)) / 3;
					else if (i == 1) normal = (normal + GetNormal(v1 - v0, addVec1 - v0)) / 3;
					else if (i == 2) normal = (normal + GetNormal(addVec2 - v0, v2 - v0)) / 3;
					else normal = (normal + GetNormal(v1 - v0, addVec2 - v0)) / 3;
					normals.Add(normal);
				}
				FVector v0 = FVector(buildings[b].layout[0].X - center.X, buildings[b].layout[0].Y - center.Y, buildings[b].height);
				FVector v1 = FVector(buildings[b].layout[1].X - center.X, buildings[b].layout[1].Y - center.Y, buildings[b].height);
				FVector v2 = FVector(buildings[b].layout[2].X - center.X, buildings[b].layout[2].Y - center.Y, buildings[b].height);
				FVector v3 = FVector(buildings[b].layout[3].X - center.X, buildings[b].layout[3].Y - center.Y, buildings[b].height);
				FVector normal1 = (GetNormal(addVec2 - addVec1, v0 - addVec1) + GetNormal(v1 - addVec1, addVec2 - addVec1) + GetNormal(v0 - addVec1, v1 - addVec1)) / 3;
				FVector normal2 = (GetNormal(addVec1 - addVec2, v2 - addVec2) + GetNormal(v3 - addVec2, addVec1 - addVec2) + GetNormal(v2 - addVec2, v3 - addVec2)) / 3;
				normals.Add(normal1);
				normals.Add(normal2);
			}
			else if (random < 0.44f) {
				//gambrel roof with ends moving inside as well
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec1 = (FVector(buildings[b].layout[0].X, buildings[b].layout[0].Y, 0.0f) - center3D) * 0.8f + FVector(0.0f, 0.0f, buildings[b].height + 5);
				FVector addVec2 = (FVector(buildings[b].layout[1].X, buildings[b].layout[1].Y, 0.0f) - center3D) * 0.8f + FVector(0.0f, 0.0f, buildings[b].height + 5);
				FVector addVec3 = (FVector(buildings[b].layout[2].X, buildings[b].layout[2].Y, 0.0f) - center3D) * 0.8f + FVector(0.0f, 0.0f, buildings[b].height + 5);
				FVector addVec4 = (FVector(buildings[b].layout[3].X, buildings[b].layout[3].Y, 0.0f) - center3D) * 0.8f + FVector(0.0f, 0.0f, buildings[b].height + 5);
				FVector addVec5 = FVector((addVec1.X + addVec2.X) / 2, (addVec1.Y + addVec2.Y) / 2, 0.0f) * 0.6f + FVector(0.0f, 0.0f, addVec1.Z + 5);
				FVector addVec6 = FVector((addVec3.X + addVec4.X) / 2, (addVec3.Y + addVec4.Y) / 2, 0.0f) * 0.6f + FVector(0.0f, 0.0f, addVec3.Z + 5);
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				vertices.Add(addVec3);
				vertices.Add(addVec4);
				vertices.Add(addVec5);
				vertices.Add(addVec6);
				tri.Add(4);
				tri.Add(8);
				tri.Add(9);
				tri.Add(4);
				tri.Add(9);
				tri.Add(5);
				tri.Add(5);
				tri.Add(9);
				tri.Add(10);
				tri.Add(5);
				tri.Add(10);
				tri.Add(6);
				tri.Add(6);
				tri.Add(10);
				tri.Add(11);
				tri.Add(6);
				tri.Add(11);
				tri.Add(7);
				tri.Add(7);
				tri.Add(11);
				tri.Add(8);
				tri.Add(7);
				tri.Add(8);
				tri.Add(4);
				tri.Add(8);
				tri.Add(12);
				tri.Add(9);
				tri.Add(9);
				tri.Add(12);
				tri.Add(13);
				tri.Add(9);
				tri.Add(13);
				tri.Add(10);
				tri.Add(10);
				tri.Add(13);
				tri.Add(11);
				tri.Add(11);
				tri.Add(13);
				tri.Add(12);
				tri.Add(11);
				tri.Add(12);
				tri.Add(8);
				for (int i = 0; i < mod; i++) {
					normals.Add(FVector());
					normals.Add(FVector());
				}
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal1 = GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3);
					FVector normal2;
					if (i == 0) {
						normal1 = (normal1 + GetNormal(addVec1 - v0, v2 - v0) + GetNormal(v1 - v0, addVec1 - v0)) / 4;
						normal2 = (GetNormal(v0 - addVec1, addVec2 - addVec1) + GetNormal(addVec2 - addVec1, addVec5 - addVec1) + GetNormal(addVec5 - addVec1, addVec4 - addVec1) + GetNormal(addVec4 - addVec1, v0 - addVec1)) / 4;
					}
					else if (i == 1) {
						normal1 = (normal1 + GetNormal(v1 - v0, addVec2 - v0) + GetNormal(addVec2 - v0, v2 - v0)) / 4;
						normal2 = (GetNormal(v0 - addVec2, addVec3 - addVec2) + GetNormal(addVec3 - addVec2, addVec5 - addVec2) + GetNormal(addVec5 - addVec2, addVec1 - addVec2) + GetNormal(addVec1 - addVec2, v0 - addVec2)) / 4;
					}
					else if (i == 2) {
						normal1 = (normal1 + GetNormal(v1 - v0, addVec3 - v0) + GetNormal(addVec3 - v0, v2 - v0)) / 4;
						normal2 = (GetNormal(v0 - addVec3, addVec4 - addVec3) + GetNormal(addVec4 - addVec3, addVec6 - addVec3) + GetNormal(addVec6 - addVec3, addVec2 - addVec3) + GetNormal(addVec2 - addVec3, v0 - addVec3)) / 4;
					}
					else {
						normal1 = (normal1 + GetNormal(v1 - v0, addVec4 - v0) + GetNormal(addVec4 - v0, v2 - v0)) / 4;
						normal2 = (GetNormal(v0 - addVec4, addVec1 - addVec4) + GetNormal(addVec1 - addVec4, addVec6 - addVec4) + GetNormal(addVec6 - addVec4, addVec3 - addVec4) + GetNormal(addVec3 - addVec4, v0 - addVec4)) / 4;
					}
					normals[4 + i] = normal1;
					normals[8 + i] = normal2;
				}
				FVector normal1 = (GetNormal(addVec1 - addVec5, addVec2 - addVec5) + GetNormal(addVec2 - addVec5, addVec6 - addVec5) + GetNormal(addVec6 - addVec5, addVec1 - addVec5)) / 3;
				FVector normal2 = (GetNormal(addVec3 - addVec6, addVec4 - addVec6) + GetNormal(addVec4 - addVec6, addVec5 - addVec6) + GetNormal(addVec5 - addVec6, addVec3 - addVec6)) / 3;
				normals.Add(normal1);
				normals.Add(normal2);
			}
			else if (random < 0.56f) {
				//gambrel roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());*/
				FVector localDir = FVector((buildings[b].layout[1] - buildings[b].layout[0]).X, (buildings[b].layout[1] - buildings[b].layout[0]).Y, 0.0f);
				float size = localDir.Size() * 0.2f;
				localDir.Normalize();
				FVector addVec1 = FVector(buildings[b].layout[0].X - center.X, buildings[b].layout[0].Y - center.Y, buildings[b].height + 5) + localDir * size;
				FVector addVec2 = FVector(buildings[b].layout[1].X - center.X, buildings[b].layout[1].Y - center.Y, buildings[b].height + 5) - localDir * size;
				FVector addVec3 = FVector(buildings[b].layout[2].X - center.X, buildings[b].layout[2].Y - center.Y, buildings[b].height + 5) - localDir * size;
				FVector addVec4 = FVector(buildings[b].layout[3].X - center.X, buildings[b].layout[3].Y - center.Y, buildings[b].height + 5) + localDir * size;
				FVector addVec5 = FVector((addVec1.X + addVec2.X) / 2, (addVec1.Y + addVec2.Y) / 2, addVec1.Z + 5);
				FVector addVec6 = FVector((addVec3.X + addVec4.X) / 2, (addVec3.Y + addVec4.Y) / 2, addVec3.Z + 5);
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				vertices.Add(addVec3);
				vertices.Add(addVec4);
				vertices.Add(addVec5);
				vertices.Add(addVec6);
				tri.Add(4);
				tri.Add(8);
				tri.Add(9);
				tri.Add(4);
				tri.Add(9);
				tri.Add(5);
				tri.Add(5);
				tri.Add(9);
				tri.Add(10);
				tri.Add(5);
				tri.Add(10);
				tri.Add(6);
				tri.Add(6);
				tri.Add(10);
				tri.Add(11);
				tri.Add(6);
				tri.Add(11);
				tri.Add(7);
				tri.Add(7);
				tri.Add(11);
				tri.Add(8);
				tri.Add(7);
				tri.Add(8);
				tri.Add(4);
				tri.Add(8);
				tri.Add(12);
				tri.Add(9);
				tri.Add(9);
				tri.Add(12);
				tri.Add(13);
				tri.Add(9);
				tri.Add(13);
				tri.Add(10);
				tri.Add(10);
				tri.Add(13);
				tri.Add(11);
				tri.Add(11);
				tri.Add(13);
				tri.Add(12);
				tri.Add(11);
				tri.Add(12);
				tri.Add(8);
				for (int i = 0; i < mod; i++) {
					normals.Add(FVector());
					normals.Add(FVector());
				}
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal1 = GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3);
					FVector normal2;
					if (i == 0) {
						normal1 = (normal1 + GetNormal(addVec1 - v0, v2 - v0) + GetNormal(v1 - v0, addVec1 - v0)) / 4;
						normal2 = (GetNormal(v0 - addVec1, addVec2 - addVec1) + GetNormal(addVec2 - addVec1, addVec5 - addVec1) + GetNormal(addVec5 - addVec1, addVec4 - addVec1) + GetNormal(addVec4 - addVec1, v0 - addVec1)) / 4;
					}
					else if (i == 1) {
						normal1 = (normal1 + GetNormal(v1 - v0, addVec2 - v0) + GetNormal(addVec2 - v0, v2 - v0)) / 4;
						normal2 = (GetNormal(v0 - addVec2, addVec3 - addVec2) + GetNormal(addVec3 - addVec2, addVec5 - addVec2) + GetNormal(addVec5 - addVec2, addVec1 - addVec2) + GetNormal(addVec1 - addVec2, v0 - addVec2)) / 4;
					}
					else if (i == 2) {
						normal1 = (normal1 + GetNormal(v1 - v0, addVec3 - v0) + GetNormal(addVec3 - v0, v2 - v0)) / 4;
						normal2 = (GetNormal(v0 - addVec3, addVec4 - addVec3) + GetNormal(addVec4 - addVec3, addVec6 - addVec3) + GetNormal(addVec6 - addVec3, addVec2 - addVec3) + GetNormal(addVec2 - addVec3, v0 - addVec3)) / 4;
					}
					else {
						normal1 = (normal1 + GetNormal(v1 - v0, addVec4 - v0) + GetNormal(addVec4 - v0, v2 - v0)) / 4;
						normal2 = (GetNormal(v0 - addVec4, addVec1 - addVec4) + GetNormal(addVec1 - addVec4, addVec6 - addVec4) + GetNormal(addVec6 - addVec4, addVec3 - addVec4) + GetNormal(addVec3 - addVec4, v0 - addVec4)) / 4;
					}
					normals[4 + i] = normal1;
					normals[8 + i] = normal2;
				}
				FVector normal1 = (GetNormal(addVec1 - addVec5, addVec2 - addVec5) + GetNormal(addVec2 - addVec5, addVec6 - addVec5) + GetNormal(addVec6 - addVec5, addVec1 - addVec5)) / 3;
				FVector normal2 = (GetNormal(addVec3 - addVec6, addVec4 - addVec6) + GetNormal(addVec4 - addVec6, addVec5 - addVec6) + GetNormal(addVec5 - addVec6, addVec3 - addVec6)) / 3;
				normals.Add(normal1);
				normals.Add(normal2);
			}
			else if (random < 0.58f) {
				//cross roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());*/
				FVector localDir = FVector((buildings[b].layout[1] - buildings[b].layout[0]).X, (buildings[b].layout[1] - buildings[b].layout[0]).Y, 0.0f);
				float size = localDir.Size() * 0.2f;
				localDir.Normalize();
				FVector addVec1 = FVector((buildings[b].layout[0].X + buildings[b].layout[1].X) / 2 - center.X, (buildings[b].layout[0].Y + buildings[b].layout[1].Y) / 2 - center.Y, buildings[b].height + 10) + localDir * size;
				FVector addVec2 = FVector((buildings[b].layout[1].X + buildings[b].layout[2].X) / 2 - center.X, (buildings[b].layout[1].Y + buildings[b].layout[2].Y) / 2 - center.Y, buildings[b].height + 10) - localDir * size;
				FVector addVec3 = FVector((buildings[b].layout[2].X + buildings[b].layout[3].X) / 2 - center.X, (buildings[b].layout[2].Y + buildings[b].layout[3].Y) / 2 - center.Y, buildings[b].height + 10) + localDir * size;
				FVector addVec4 = FVector((buildings[b].layout[3].X + buildings[b].layout[0].X) / 2 - center.X, (buildings[b].layout[3].Y + buildings[b].layout[0].Y) / 2 - center.Y, buildings[b].height + 10) - localDir * size;
				FVector addVec5 = FVector(/*center.X*/0.0f, /*center.Y*/0.0f, buildings[b].height + 10);
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				vertices.Add(addVec3);
				vertices.Add(addVec4);
				vertices.Add(addVec5);
				tri.Add(4);
				tri.Add(8);
				tri.Add(5);
				tri.Add(5);
				tri.Add(8);
				tri.Add(12);
				tri.Add(5);
				tri.Add(12);
				tri.Add(9);
				tri.Add(5);
				tri.Add(9);
				tri.Add(6);
				tri.Add(6);
				tri.Add(9);
				tri.Add(12);
				tri.Add(6);
				tri.Add(12);
				tri.Add(10);
				tri.Add(6);
				tri.Add(10);
				tri.Add(7);
				tri.Add(7);
				tri.Add(10);
				tri.Add(12);
				tri.Add(7);
				tri.Add(12);
				tri.Add(11);
				tri.Add(7);
				tri.Add(11);
				tri.Add(4);
				tri.Add(4);
				tri.Add(11);
				tri.Add(12);
				tri.Add(4);
				tri.Add(12);
				tri.Add(8);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal = GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3);
					if (i == 0) normal = (normal + GetNormal(addVec1 - v0, addVec5 - v0) + GetNormal(addVec5 - v0, addVec4 - v0)) / 4;
					else if (i == 1) normal = (normal + GetNormal(addVec2 - v0, addVec5 - v0) + GetNormal(addVec5 - v0, addVec1 - v0)) / 4;
					else if (i == 2) normal = (normal + GetNormal(addVec3 - v0, addVec5 - v0) + GetNormal(addVec5 - v0, addVec2 - v0)) / 4;
					else normal = (normal + GetNormal(addVec4 - v0, addVec5 - v0) + GetNormal(addVec5 - v0, addVec3 - v0)) / 4;
					normals.Add(normal);
				}
				FVector v0 = FVector(buildings[b].layout[0].X - center.X, buildings[b].layout[0].Y - center.Y, buildings[b].height);
				FVector v1 = FVector(buildings[b].layout[1].X - center.X, buildings[b].layout[1].Y - center.Y, buildings[b].height);
				FVector v2 = FVector(buildings[b].layout[2].X - center.X, buildings[b].layout[2].Y - center.Y, buildings[b].height);
				FVector v3 = FVector(buildings[b].layout[3].X - center.X, buildings[b].layout[3].Y - center.Y, buildings[b].height);
				FVector normal1 = (GetNormal(v0 - addVec1, v1 - addVec1) + GetNormal(v1 - addVec1, addVec5 - addVec1) + GetNormal(addVec5 - addVec1, v0 - addVec1)) / 3;
				FVector normal2 = (GetNormal(v1 - addVec2, v2 - addVec2) + GetNormal(v2 - addVec2, addVec5 - addVec2) + GetNormal(addVec5 - addVec2, v1 - addVec2)) / 3;
				FVector normal3 = (GetNormal(v2 - addVec3, v3 - addVec3) + GetNormal(v3 - addVec3, addVec5 - addVec3) + GetNormal(addVec5 - addVec3, v2 - addVec3)) / 3;
				FVector normal4 = (GetNormal(v3 - addVec4, v0 - addVec4) + GetNormal(v0 - addVec4, addVec5 - addVec4) + GetNormal(addVec5 - addVec4, v3 - addVec4)) / 3;
				normals.Add(normal1);
				normals.Add(normal2);
				normals.Add(normal3);
				normals.Add(normal4);
				normals.Add(FVector().UpVector);
			}
			else if (random < 0.7f) {
				//building upon the roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());
				vertexColors.Add(vertexColors.Last());*/
				FVector localDir = FVector((buildings[b].layout[1] - buildings[b].layout[0]).X, (buildings[b].layout[1] - buildings[b].layout[0]).Y, 0.0f);
				float size = localDir.Size() * 0.2f;
				localDir.Normalize();
				FVector addVec1 = (FVector(buildings[b].layout[0].X, buildings[b].layout[0].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec2 = (FVector(buildings[b].layout[1].X, buildings[b].layout[1].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec3 = (FVector(buildings[b].layout[2].X, buildings[b].layout[2].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec4 = (FVector(buildings[b].layout[3].X, buildings[b].layout[3].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec5 = FVector(addVec1.X, addVec1.Y, addVec1.Z + 10);
				FVector addVec6 = FVector(addVec2.X, addVec2.Y, addVec2.Z + 10);
				FVector addVec7 = FVector(addVec3.X, addVec3.Y, addVec3.Z + 10);
				FVector addVec8 = FVector(addVec4.X, addVec4.Y, addVec4.Z + 10);
				FVector addVec9 = FVector((addVec1.X + addVec2.X) / 2, (addVec1.Y + addVec2.Y) / 2, addVec5.Z + 5);
				FVector addVec10 = FVector((addVec3.X + addVec4.X) / 2, (addVec3.Y + addVec4.Y) / 2, addVec7.Z + 5);
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				vertices.Add(addVec3);
				vertices.Add(addVec4);
				vertices.Add(addVec5);
				vertices.Add(addVec6);
				vertices.Add(addVec7);
				vertices.Add(addVec8);
				vertices.Add(addVec9);
				vertices.Add(addVec10);
				tri.Add(4);
				tri.Add(8);
				tri.Add(9);
				tri.Add(4);
				tri.Add(9);
				tri.Add(5);
				tri.Add(5);
				tri.Add(9);
				tri.Add(10);
				tri.Add(5);
				tri.Add(10);
				tri.Add(6);
				tri.Add(6);
				tri.Add(10);
				tri.Add(11);
				tri.Add(6);
				tri.Add(11);
				tri.Add(7);
				tri.Add(7);
				tri.Add(11);
				tri.Add(8);
				tri.Add(7);
				tri.Add(8);
				tri.Add(4);
				tri.Add(8);
				tri.Add(12);
				tri.Add(13);
				tri.Add(8);
				tri.Add(13);
				tri.Add(9);
				tri.Add(9);
				tri.Add(13);
				tri.Add(14);
				tri.Add(9);
				tri.Add(14);
				tri.Add(10);
				tri.Add(10);
				tri.Add(14);
				tri.Add(15);
				tri.Add(10);
				tri.Add(15);
				tri.Add(11);
				tri.Add(11);
				tri.Add(15);
				tri.Add(12);
				tri.Add(11);
				tri.Add(12);
				tri.Add(8);
				tri.Add(12);
				tri.Add(16);
				tri.Add(13);
				tri.Add(13);
				tri.Add(16);
				tri.Add(17);
				tri.Add(13);
				tri.Add(17);
				tri.Add(14);
				tri.Add(14);
				tri.Add(17);
				tri.Add(15);
				tri.Add(15);
				tri.Add(17);
				tri.Add(16);
				tri.Add(15);
				tri.Add(16);
				tri.Add(12);
				for (int i = 0; i < mod; i++) {
					normals.Add(FVector());
					normals.Add(FVector());
					normals.Add(FVector());
				}
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal1 = (GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3) + FVector().UpVector) / 3;
					FVector normal2 = FVector().UpVector;
					FVector normal3;
					if (i == 0) {
						normal2 = (normal2 + GetNormal(addVec2 - addVec1, addVec5 - addVec1) + GetNormal(addVec5 - addVec1, addVec4 - addVec1)) / 3;
						normal3 = (GetNormal(addVec1 - addVec5, addVec9 - addVec5) + GetNormal(addVec9 - addVec5, addVec8 - addVec5) + GetNormal(addVec8 - addVec5, addVec1 - addVec5)) / 3;
					}
					else if (i == 1) {
						normal2 = (normal2 + GetNormal(addVec3 - addVec2, addVec6 - addVec2) + GetNormal(addVec6 - addVec2, addVec1 - addVec2)) / 3;
						normal3 = (GetNormal(addVec2 - addVec6, addVec7 - addVec6) + GetNormal(addVec7 - addVec6, addVec9 - addVec6) + GetNormal(addVec9 - addVec6, addVec2 - addVec6)) / 3;
					}
					else if (i == 2) {
						normal2 = (normal2 + GetNormal(addVec4 - addVec3, addVec7 - addVec3) + GetNormal(addVec7 - addVec3, addVec2 - addVec3)) / 3;
						normal3 = (GetNormal(addVec3 - addVec7, addVec10 - addVec7) + GetNormal(addVec10 - addVec7, addVec6 - addVec7) + GetNormal(addVec6 - addVec7, addVec3 - addVec7)) / 3;
					}
					else {
						normal2 = (normal2 + GetNormal(addVec1 - addVec4, addVec8 - addVec4) + GetNormal(addVec8 - addVec4, addVec3 - addVec4)) / 3;
						normal3 = (GetNormal(addVec4 - addVec8, addVec5 - addVec8) + GetNormal(addVec5 - addVec8, addVec10 - addVec8) + GetNormal(addVec10 - addVec8, addVec4 - addVec8)) / 3;
					}
					normals[4 + i] = normal1;
					normals[8 + i] = normal2;
					normals[12 + i] = normal3;
				}
				FVector normal1 = (GetNormal(addVec5 - addVec9, addVec6 - addVec9) + GetNormal(addVec6 - addVec9, addVec10 - addVec9) + GetNormal(addVec10 - addVec9, addVec5 - addVec9)) / 3;
				FVector normal2 = (GetNormal(addVec7 - addVec10, addVec8 - addVec10) + GetNormal(addVec8 - addVec10, addVec9 - addVec10) + GetNormal(addVec9 - addVec10, addVec7 - addVec10)) / 3;
				normals.Add(normal1);
				normals.Add(normal2);
			}
			else {
				//flat roof
				tri.Add(4);
				tri.Add(6);
				tri.Add(5);
				tri.Add(4);
				tri.Add(7);
				tri.Add(6);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, 0.0f);
					FVector normal = (GetNormal(v3 - v0, v1 - v0) + GetNormal(v2 - v0, v3 - v0) + FVector().UpVector) / 3;
					normals.Add(normal);
				}
			}
			break;
		case 5:
			if (random < 0.2f) {
				//pyramid roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec = FVector(0.0f, 0.0f, buildings[b].height + 10);
				vertices.Add(addVec);
				tri.Add(5);
				tri.Add(10);
				tri.Add(6);
				tri.Add(6);
				tri.Add(10);
				tri.Add(7);
				tri.Add(7);
				tri.Add(10);
				tri.Add(8);
				tri.Add(8);
				tri.Add(10);
				tri.Add(9);
				tri.Add(9);
				tri.Add(10);
				tri.Add(5);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal = (GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3) + GetNormal(addVec - v0, v2 - v0) + GetNormal(v1 - v0, addVec - v0)) / 4;
					normals.Add(normal);
				}
				normals.Add(FVector().UpVector);
			}
			else if (random < 0.5f) {
				//building upon roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec1 = (FVector(buildings[b].layout[0].X, buildings[b].layout[0].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec2 = (FVector(buildings[b].layout[1].X, buildings[b].layout[1].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec3 = (FVector(buildings[b].layout[2].X, buildings[b].layout[2].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec4 = (FVector(buildings[b].layout[3].X, buildings[b].layout[3].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec5 = (FVector(buildings[b].layout[4].X, buildings[b].layout[4].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec6 = FVector(addVec1.X, addVec1.Y, buildings[b].height + 10);
				FVector addVec7 = FVector(addVec2.X, addVec2.Y, buildings[b].height + 10);
				FVector addVec8 = FVector(addVec3.X, addVec3.Y, buildings[b].height + 10);
				FVector addVec9 = FVector(addVec4.X, addVec4.Y, buildings[b].height + 10);
				FVector addVec10 = FVector(addVec5.X, addVec5.Y, buildings[b].height + 10);
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				vertices.Add(addVec3);
				vertices.Add(addVec4);
				vertices.Add(addVec5);
				vertices.Add(addVec6);
				vertices.Add(addVec7);
				vertices.Add(addVec8);
				vertices.Add(addVec9);
				vertices.Add(addVec10);
				tri.Add(5);
				tri.Add(10);
				tri.Add(11);
				tri.Add(5);
				tri.Add(11);
				tri.Add(6);
				tri.Add(6);
				tri.Add(11);
				tri.Add(12);
				tri.Add(6);
				tri.Add(12);
				tri.Add(7);
				tri.Add(7);
				tri.Add(12);
				tri.Add(13);
				tri.Add(7);
				tri.Add(13);
				tri.Add(8);
				tri.Add(8);
				tri.Add(13);
				tri.Add(14);
				tri.Add(8);
				tri.Add(14);
				tri.Add(9);
				tri.Add(9);
				tri.Add(14);
				tri.Add(10);
				tri.Add(9);
				tri.Add(10);
				tri.Add(5);
				tri.Add(10);
				tri.Add(15);
				tri.Add(16);
				tri.Add(10);
				tri.Add(16);
				tri.Add(11);
				tri.Add(11);
				tri.Add(16);
				tri.Add(17);
				tri.Add(11);
				tri.Add(17);
				tri.Add(12);
				tri.Add(12);
				tri.Add(17);
				tri.Add(18);
				tri.Add(12);
				tri.Add(18);
				tri.Add(13);
				tri.Add(13);
				tri.Add(18);
				tri.Add(19);
				tri.Add(13);
				tri.Add(19);
				tri.Add(14);
				tri.Add(14);
				tri.Add(19);
				tri.Add(15);
				tri.Add(14);
				tri.Add(15);
				tri.Add(10);
				tri.Add(15);
				tri.Add(17);
				tri.Add(16);
				tri.Add(15);
				tri.Add(18);
				tri.Add(17);
				tri.Add(15);
				tri.Add(19);
				tri.Add(18);
				for (int i = 0; i < mod; i++) {
					normals.Add(FVector());
					normals.Add(FVector());
					normals.Add(FVector());
				}
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal1 = (GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3) + FVector().UpVector) / 3;
					FVector normal2 = FVector().UpVector;
					if (i == 0) normal2 = (normal2 + GetNormal(addVec2 - addVec1, addVec6 - addVec1) + GetNormal(addVec6 - addVec1, addVec5 - addVec1)) / 3;
					else if (i == 1) normal2 = (normal2 + GetNormal(addVec3 - addVec2, addVec7 - addVec2) + GetNormal(addVec7 - addVec2, addVec1 - addVec2)) / 3;
					else if (i == 2) normal2 = (normal2 + GetNormal(addVec4 - addVec3, addVec8 - addVec3) + GetNormal(addVec8 - addVec3, addVec2 - addVec3)) / 3;
					else if (i == 3) normal2 = (normal2 + GetNormal(addVec5 - addVec4, addVec9 - addVec4) + GetNormal(addVec9 - addVec4, addVec3 - addVec4)) / 3;
					else normal2 = (normal2 + GetNormal(addVec1 - addVec5, addVec10 - addVec5) + GetNormal(addVec10 - addVec5, addVec4 - addVec5)) / 3;
					normals[5 + i] = normal1;
					normals[10 + i] = normal2;
					normals[15 + i] = normal2;
				}
			}
			else if (random < 0.75f) {
				//pyramid building upon roof
				/*normals.Add(FVector(1.0f, 0.0f, 0.0f));
				uV0s.Add(FVector2D(0.0f, 0.0f));
				vertexColors.Add(vertexColors.Last());*/
				FVector addVec1 = (FVector(buildings[b].layout[0].X, buildings[b].layout[0].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec2 = (FVector(buildings[b].layout[1].X, buildings[b].layout[1].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec3 = (FVector(buildings[b].layout[2].X, buildings[b].layout[2].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec4 = (FVector(buildings[b].layout[3].X, buildings[b].layout[3].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec5 = (FVector(buildings[b].layout[4].X, buildings[b].layout[4].Y, 0.0f) - center3D) * 0.8f + center3D + FVector(-center.X, -center.Y, buildings[b].height);
				FVector addVec6 = FVector(addVec1.X, addVec1.Y, buildings[b].height + 10);
				FVector addVec7 = FVector(addVec2.X, addVec2.Y, buildings[b].height + 10);
				FVector addVec8 = FVector(addVec3.X, addVec3.Y, buildings[b].height + 10);
				FVector addVec9 = FVector(addVec4.X, addVec4.Y, buildings[b].height + 10);
				FVector addVec10 = FVector(addVec5.X, addVec5.Y, buildings[b].height + 10);
				FVector addVec11 = FVector(0.0f, 0.0f, addVec6.Z + 10);
				vertices.Add(addVec1);
				vertices.Add(addVec2);
				vertices.Add(addVec3);
				vertices.Add(addVec4);
				vertices.Add(addVec5);
				vertices.Add(addVec6);
				vertices.Add(addVec7);
				vertices.Add(addVec8);
				vertices.Add(addVec9);
				vertices.Add(addVec10);
				vertices.Add(addVec11);
				tri.Add(5);
				tri.Add(10);
				tri.Add(11);
				tri.Add(5);
				tri.Add(11);
				tri.Add(6);
				tri.Add(6);
				tri.Add(11);
				tri.Add(12);
				tri.Add(6);
				tri.Add(12);
				tri.Add(7);
				tri.Add(7);
				tri.Add(12);
				tri.Add(13);
				tri.Add(7);
				tri.Add(13);
				tri.Add(8);
				tri.Add(8);
				tri.Add(13);
				tri.Add(14);
				tri.Add(8);
				tri.Add(14);
				tri.Add(9);
				tri.Add(9);
				tri.Add(14);
				tri.Add(10);
				tri.Add(9);
				tri.Add(10);
				tri.Add(5);
				tri.Add(10);
				tri.Add(15);
				tri.Add(16);
				tri.Add(10);
				tri.Add(16);
				tri.Add(11);
				tri.Add(11);
				tri.Add(16);
				tri.Add(17);
				tri.Add(11);
				tri.Add(17);
				tri.Add(12);
				tri.Add(12);
				tri.Add(17);
				tri.Add(18);
				tri.Add(12);
				tri.Add(18);
				tri.Add(13);
				tri.Add(13);
				tri.Add(18);
				tri.Add(19);
				tri.Add(13);
				tri.Add(19);
				tri.Add(14);
				tri.Add(14);
				tri.Add(19);
				tri.Add(15);
				tri.Add(14);
				tri.Add(15);
				tri.Add(10);
				tri.Add(15);
				tri.Add(20);
				tri.Add(16);
				tri.Add(16);
				tri.Add(20);
				tri.Add(17);
				tri.Add(17);
				tri.Add(20);
				tri.Add(18);
				tri.Add(18);
				tri.Add(20);
				tri.Add(19);
				tri.Add(19);
				tri.Add(20);
				tri.Add(15);
				for (int i = 0; i < mod; i++) {
					normals.Add(FVector());
					normals.Add(FVector());
					normals.Add(FVector());
				}
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = -FVector().UpVector;
					FVector normal1 = (GetNormal(v3, v1 - v0) + GetNormal(v2 - v0, v3) + FVector().UpVector) / 3;
					FVector normal2 = FVector().UpVector;
					FVector normal3;
					if (i == 0) {
						normal2 = (normal2 + GetNormal(addVec2 - addVec1, addVec6 - addVec1) + GetNormal(addVec6 - addVec1, addVec5 - addVec1)) / 3;
						normal3 = (GetNormal(addVec1 - addVec6, addVec7 - addVec6) + GetNormal(addVec7 - addVec6, addVec11 - addVec6) + GetNormal(addVec11 - addVec6, addVec10 - addVec6) + GetNormal(addVec10 - addVec6, addVec1 - addVec6)) / 4;
					}
					else if (i == 1) {
						normal2 = (normal2 + GetNormal(addVec3 - addVec2, addVec7 - addVec2) + GetNormal(addVec7 - addVec2, addVec1 - addVec2)) / 3;
						normal3 = (GetNormal(addVec2 - addVec7, addVec8 - addVec7) + GetNormal(addVec8 - addVec7, addVec11 - addVec7) + GetNormal(addVec11 - addVec7, addVec6 - addVec7) + GetNormal(addVec6 - addVec7, addVec2 - addVec7)) / 4;
					}
					else if (i == 2) {
						normal2 = (normal2 + GetNormal(addVec4 - addVec3, addVec8 - addVec3) + GetNormal(addVec8 - addVec3, addVec2 - addVec3)) / 3;
						normal3 = (GetNormal(addVec3 - addVec8, addVec9 - addVec8) + GetNormal(addVec9 - addVec8, addVec11 - addVec8) + GetNormal(addVec11 - addVec8, addVec7 - addVec8) + GetNormal(addVec7 - addVec8, addVec3 - addVec8)) / 4;
					}
					else if (i == 3) {
						normal2 = (normal2 + GetNormal(addVec5 - addVec4, addVec9 - addVec4) + GetNormal(addVec9 - addVec4, addVec3 - addVec4)) / 3;
						normal3 = (GetNormal(addVec4 - addVec9, addVec10 - addVec9) + GetNormal(addVec10 - addVec9, addVec11 - addVec9) + GetNormal(addVec11 - addVec9, addVec8 - addVec9) + GetNormal(addVec8 - addVec9, addVec4 - addVec9)) / 4;
					}
					else {
						normal2 = (normal2 + GetNormal(addVec1 - addVec5, addVec10 - addVec5) + GetNormal(addVec10 - addVec5, addVec4 - addVec5)) / 3;
						normal3 = (GetNormal(addVec5 - addVec10, addVec6 - addVec10) + GetNormal(addVec6 - addVec10, addVec11 - addVec10) + GetNormal(addVec11 - addVec10, addVec9 - addVec10) + GetNormal(addVec9 - addVec10, addVec5 - addVec10)) / 4;
					}
					normals[5 + i] = normal1;
					normals[10 + i] = normal2;
					normals[15 + i] = normal3;
				}
				normals.Add(FVector().UpVector);
			}
			else {
				//flat roof
				tri.Add(5);
				tri.Add(7);
				tri.Add(6);
				tri.Add(5);
				tri.Add(8);
				tri.Add(7);
				tri.Add(5);
				tri.Add(9);
				tri.Add(8);
				for (int i = 0; i < mod; i++)
				{
					FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
					FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
					FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
					FVector v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, 0.0f);
					FVector normal = (GetNormal(v3 - v0, v1 - v0) + GetNormal(v2 - v0, v3 - v0) + FVector().UpVector) / 3;
					normals.Add(normal);
				}
			}
			break;
		case 6:
			for (int i = 0; i < mod; i++)
			{
				FVector v0 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, buildings[b].height);
				FVector v1 = FVector(buildings[b].layout[(i + 1) % mod].X - center.X, buildings[b].layout[(i + 1) % mod].Y - center.Y, buildings[b].height);
				FVector v2 = FVector(buildings[b].layout[(i + mod - 1) % mod].X - center.X, buildings[b].layout[(i + mod - 1) % mod].Y - center.Y, buildings[b].height);
				FVector v3 = FVector(buildings[b].layout[i % mod].X - center.X, buildings[b].layout[i % mod].Y - center.Y, 0.0f);
				FVector normal = (GetNormal(v3 - v0, v1 - v0) + GetNormal(v2 - v0, v3 - v0) + FVector().UpVector) / 3;
				normals.Add(normal);
			}
			break;
		default:
			break;
		}

		//TODO remove when found solution how to handle correct setup
		uV0s.Empty();
		vertexColors.Empty();

		ABuildingActor* newBuilding = GetWorld()->SpawnActor<ABuildingActor>(center3D, FRotator(0.0f, 0.0f, 0.0f), FActorSpawnParameters());
		newBuilding->ConstructBuilding(tri, vertices, normals, uV0s, vertexColors, material);
	}
}

void AStreetGenerator::VisualizeStreets()
{
	TArray<Segment> streets = qT_Street.GetAllObjects();
	for (int s = 0; s < streets.Num(); s++)
	{
		TArray<int32> tri;
		TArray<FVector> vertices;
		TArray<FVector> normals;
		TArray<FVector2D> uV0s;
		TArray<FLinearColor> vertexColors;
		UMaterial* material = materials.Last();
		FVector2D center = (streets[s].GetStart() + streets[s].GetEnd()) / 2;
		FVector center3D = FVector(center.X, center.Y, 0.0f);
		FVector2D direction = streets[s].direction.GetRotated(90.0f);
		FVector direction3D = FVector(direction.X, direction.Y, 0.0f);
		float size;

		if (streets[s].isHighway) {
			size = config->highway_width / 2.0f;
			material = materials[1];
		}
		else {
			size = config->sub_width / 2.0f;
			material = materials[3];
		}
		FVector v1 = FVector(streets[s].GetStart().X, streets[s].GetStart().Y, 1.0f) + direction3D * size;
		FVector v2 = FVector(streets[s].GetEnd().X, streets[s].GetEnd().Y, 1.0f) + direction3D * size;
		FVector v3 = FVector(streets[s].GetEnd().X, streets[s].GetEnd().Y, 1.0f) - direction3D * size;
		FVector v4 = FVector(streets[s].GetStart().X, streets[s].GetStart().Y, 1.0f) - direction3D * size;
		vertices.Add(v1 - center3D);
		vertices.Add(v2 - center3D);
		vertices.Add(v3 - center3D);
		vertices.Add(v4 - center3D);
		tri.Add(0);
		tri.Add(1);
		tri.Add(3);
		tri.Add(3);
		tri.Add(1);
		tri.Add(2);
		normals.Init(FVector().UpVector, 4);

		ABuildingActor* newStreet = GetWorld()->SpawnActor<ABuildingActor>(center3D, FRotator(0.0f, 0.0f, 0.0f), FActorSpawnParameters());
		newStreet->ConstructBuilding(tri, vertices, normals, uV0s, vertexColors, material);
	}
}

void AStreetGenerator::ConstructCity()
{
	MakeNoiseP();
	CreateInitialSetup();
	while (activSegments.Num() > 0)
	{
		Generate();
	}
	if(showBuildings) VisualizeBuildings();
	if(showStreets) VisualizeStreets();
}

TArray<FBuildingSegment> AStreetGenerator::DrawQuadtree(QuadTree<Building> qT)
{
	TArray<FBuildingSegment> returnValues;
	Bounds b = qT.GetBounds();
	TArray<FVector2D> layout;
	layout.Add(FVector2D(b.x, b.y));
	layout.Add(FVector2D(b.x + b.width, b.y));
	layout.Add(FVector2D(b.x + b.width, b.y + b.height));
	layout.Add(FVector2D(b.x, b.y + b.height));
	returnValues.Add(FBuildingSegment(layout, 1.0f, EBuildingType::BT_Residental));
	TArray<QuadTree<Building>> nodes = qT.GetNodes();
	if (nodes[0].GetDepth() != -1) {
		for (int i = 0; i < nodes.Num(); i++)
		{
			returnValues.Append(DrawQuadtree(nodes[i]));
		}
	}
	return returnValues;
}

EBuildingType AStreetGenerator::GetBuildingType(FVector2D center)
{
	EBuildingType type;
	float value = GetNoiseValue(center.X, center.Y);
	if (value < 0.2f) type = EBuildingType::BT_Park;
	else if (value < 0.4f) type = EBuildingType::BT_Industrial;
	else if (value < 0.6f) {
		float random = FRandomStream(center.X + center.Y).FRand();
		float chance = (value - 0.4f) / 0.2f;
		if (random > chance) type = EBuildingType::BT_Industrial;
		else type = EBuildingType::BT_Commercial;
		/*float random = FRandomStream(center.X + center.Y).FRand();
		float chance = (value - 0.4f) / 0.2f;
		if(random > chance) return EBuildingType::BT_Industrial;
		else return EBuildingType::BT_Commercial;*/
	}
	else if (value < 0.8f) {
		float random = FRandomStream(center.X + center.Y).FRand();
		float chance = (value - 0.6f) / 0.2f;
		if (random > chance) type = EBuildingType::BT_Commercial;
		else type = EBuildingType::BT_Residental;
	}
	else type = EBuildingType::BT_Residental;

	return type;
	/*float random = FRandomStream(center.X + center.Y).FRand();
	switch (type)
	{
	case EBuildingType::BT_Residental:
		if (random < 0.3f) return EBuildingType::BT_Park;
		else return type;
	case EBuildingType::BT_Industrial:
		return type;
	case EBuildingType::BT_Commercial:
		if (random < 0.5f) return EBuildingType::BT_Park;
		else return type;
	case EBuildingType::BT_Park:
		return type;
	default:
		return EBuildingType::BT_Park;
	}*/
}

float AStreetGenerator::DecideHeight(EBuildingType buildingType, FVector2D center)
{
	switch (buildingType)
	{
	case EBuildingType::BT_Park:
		return 0.0f;
	case EBuildingType::BT_Industrial:
		return FRandomStream(center.X + center.Y).FRandRange(config->industrialHeightMin, config->industrialHeightMax);
	case EBuildingType::BT_Commercial:
		return FRandomStream(center.X + center.Y).FRandRange(config->commercialHeightMin, config->commercialHeightMax);
	case EBuildingType::BT_Residental:
		return FRandomStream(center.X + center.Y).FRandRange(config->residentalHeightMin, config->residentalHeightMax);
	default:
		return -1.0f;
	}
}

FVector AStreetGenerator::GetNormal(FVector v1, FVector v2)
{
	FVector result = FVector::CrossProduct(v1, v2);
	result.Normalize();
	return result;
}

bool AStreetGenerator::AdjustSegment(Segment & segment, bool positiveDirection, bool reset)
{
	float minLength;
	if (segment.isHighway) minLength = config->highway_minLength;
	else minLength = config->sub_minLength;
	
	if (GetNoiseValue(segment.GetEnd().X, segment.GetEnd().Y) < config->minGrowthValue) {
		if (segment.length - 1 < minLength) {
			FVector2D temp;
			if (reset) temp = segment.GetOriginalDirection() * segment.GetOriginalLength();
			else temp = segment.direction * segment.GetOriginalLength();
			float rotationAngle;
			float maxDeviation;
			//double the precision of GlobalGoals
			if (segment.isHighway) {
				maxDeviation = config->highway_maxDeviationAngle;
				rotationAngle = maxDeviation / config->highway_sampleAmount;
			}
			else {
				maxDeviation = config->sub_maxDeviationAngle;
				rotationAngle = maxDeviation / config->sub_sampleAmount;
			}
			if (!positiveDirection) rotationAngle *= -1;
			FVector2D newDirection = temp.GetRotated(rotationAngle);
			FVector2D newEnd = segment.GetStart() + newDirection;
			newDirection.Normalize();
			float midAngle = FMath::Acos(FVector2D::DotProduct(segment.GetOriginalDirection(), newDirection)) * 180 / PI;
			if (midAngle > maxDeviation) {
				if (positiveDirection) return AdjustSegment(segment, false, true);
				else return false;
			}
			else {
				segment.SetEnd(newEnd);
				return AdjustSegment(segment, positiveDirection);
			}
		}
		else {
			segment.SetEnd(segment.GetEnd() - segment.direction);
			return AdjustSegment(segment, positiveDirection);
		}
	}
	else return true;
}

float AStreetGenerator::GetNoiseValue(int x, int y)
{
	float value;
	//if value is outside of desired area
	if (x<0 || x >= areaWidth || y<0 || y >= areaHeight) value = pN.OctaveNoise((double)x / areaWidth * shiftX, (double)y / areaHeight * shiftY, octaves, persistence);
	else value = noise[x * areaWidth + y];
	return value;
}

float AStreetGenerator::GetHeight(int x, int y)
{
	float value;
	//if value is outside of desired area
	if (x<0 || x >= areaWidth || y<0 || y >= areaHeight) value = pNHeight.OctaveNoise((double)x / areaWidth * shiftX, (double)y / areaHeight * shiftY, octaves, persistence);
	else value = height[x * areaWidth + y];
	return value;
}

TArray<FVector2D> AStreetGenerator::GetStartpoints()
{
	TArray<FVector2D> startPoints;
	for (int x = 0; x < areaWidth; x++)
	{
		for (int y = 0; y < areaHeight; y++)
		{
			if (GetNoiseValue(x, y) == 1.0f) {
				bool valid = true;
				for (int i = 0; i < startPoints.Num(); i++)
				{
					if ((FVector2D(x, y) - startPoints[i]).Size() < config->minDistanceBetweenStartPoints) {
						valid = false;
						break;
					}
				}
				if (valid) startPoints.Add(FVector2D(x, y));
			}
		}
	}
	return startPoints;
}

Segment AStreetGenerator::SplitSegment(Segment &toSplit, FVector2D splitLocation)
{
	Segment splitPart = Segment(toSplit.GetStart(), splitLocation, toSplit.isHighway, toSplit.category);
	toSplit.GetStart() = splitLocation;
	return splitPart;
}

void AStreetGenerator::UpdateBuildings(Segment newSegment, FVector2D rotDirection)
{
	/*TArray<FVector2D> layout;
	layout.Add(FVector2D(newSegment.start - rotDirection));
	layout.Add(FVector2D(newSegment.start + rotDirection));
	layout.Add(FVector2D(newSegment.end + rotDirection));
	layout.Add(FVector2D(newSegment.end - rotDirection));*/
	FVector2D mid = (newSegment.GetEnd() + newSegment.GetStart()) / 2.0f;
	float height = GetNoiseValue(mid.X, mid.Y);
	TArray<Building> allBuildings = qT_Building.GetAllObjects();
	TArray<Building> buildings = qT_Building.GetAllObjects();// GetObjectsInNode(Building(layout, height, EBuildingType::BT_Residental)); //GetAllObjects() to clear some minor errors
	TArray<Building> newBuildings;
	for (int b = 0; b < buildings.Num(); b++)
	{
		allBuildings.Remove(buildings[b]);
		float width;
		if (newSegment.isHighway) width = config->highway_width / 2.0f;
		else width = config->sub_width / 2.0f;
		Building newBuilding = newSegment.CheckIntersectionBuilding(buildings[b], true, config->minBuildingArea, width);
		if (newBuilding.layout.Num() > 0) newBuildings.Add(newBuilding);
		if (buildings[b].GetPolygonArea() < config->minBuildingArea) buildings.RemoveAt(b);
	}
	qT_Building.Clear();
	allBuildings.Append(buildings);
	allBuildings.Append(newBuildings);
	for (int i = 0; i < allBuildings.Num(); i++)
	{
		qT_Building.AddSegment(allBuildings[i]);
	}
}

void AStreetGenerator::ConstructBuildings(Segment newSegment, FVector2D rotDirection, float height)
{
	FVector2D offset = rotDirection;
	offset.Normalize();
	if (newSegment.isHighway) offset *= config->highway_width / 2.0f;
	else offset *= config->sub_width / 2.0f;
	TArray<FVector2D> newBuildingLayoutR;
	TArray<FVector2D> newBuildingLayoutL;
	newBuildingLayoutR.Add(newSegment.GetStart() + offset + newSegment.direction);//offset so less checks need to be done and less conflicts are created
	newBuildingLayoutR.Add(newSegment.GetStart() + rotDirection + newSegment.direction);
	newBuildingLayoutR.Add(newSegment.GetEnd() + rotDirection - newSegment.direction);
	newBuildingLayoutR.Add(newSegment.GetEnd() + offset - newSegment.direction);
	newBuildingLayoutL.Add(newSegment.GetEnd() - offset - newSegment.direction);
	newBuildingLayoutL.Add(newSegment.GetEnd() - rotDirection - newSegment.direction);
	newBuildingLayoutL.Add(newSegment.GetStart() - rotDirection + newSegment.direction);
	newBuildingLayoutL.Add(newSegment.GetStart() - offset + newSegment.direction);
	TArray<Segment> streetsR = qT_Street.GetObjectsInNode(Segment(newSegment.GetStart(), newSegment.GetEnd() + rotDirection, false));
	TArray<Segment> streetsL = qT_Street.GetObjectsInNode(Segment(newSegment.GetStart(), newSegment.GetEnd() - rotDirection, false));
	FVector2D centerR = (newBuildingLayoutR[0] + newBuildingLayoutR[2]) / 2;
	FVector2D centerL = (newBuildingLayoutL[0] + newBuildingLayoutL[2]) / 2;
	EBuildingType buildingTypeR = GetBuildingType(centerR);
	EBuildingType buildingTypeL = GetBuildingType(centerL);
	float heightR = DecideHeight(buildingTypeR, centerR) * height;
	float heightL = DecideHeight(buildingTypeL, centerL) * height;
	Building buildingR = Building(newBuildingLayoutR, heightR, buildingTypeR);
	Building buildingL = Building(newBuildingLayoutL, heightL, buildingTypeL);
	TArray<Building> buildingsR = qT_Building.GetAllObjects();// GetObjectsInNode(buildingR);
	TArray<Building> buildingsL = qT_Building.GetAllObjects();// GetObjectsInNode(buildingL);
	//1. is building cut by existing street?
	//2. is building blocked by existing building?
	//if everthing went well, add new buildings

	bool valid = true;
	for (int r = 0; r < streetsR.Num(); r++)
	{
		streetsR[r].CheckIntersectionBuilding(buildingR, false, config->minBuildingArea, offset.Size());
	}
	for (int br = 0; br < buildingsR.Num(); br++)
	{
		if (buildingR.CheckForIntersection(buildingsR[br])) valid = false;
	}
	if (valid && buildingR.GetPolygonArea() > config->minBuildingArea) qT_Building.AddSegment(buildingR);

	valid = true;
	for (int l = 0; l < streetsL.Num(); l++)
	{
		streetsL[l].CheckIntersectionBuilding(buildingL, false, config->minBuildingArea, offset.Size());
	}
	for (int bl = 0; bl < buildingsL.Num(); bl++)
	{
		if (buildingL.CheckForIntersection(buildingsL[bl])) valid = false;
	}
	if (valid && buildingL.GetPolygonArea() > config->minBuildingArea) qT_Building.AddSegment(buildingL);
}

bool AStreetGenerator::GetIntersection(FStreetSegment s1, FStreetSegment s2)
{
	Segment a = Segment(s1.start, s1.end, s1.isHighway);
	Segment b = Segment(s2.start, s2.end, s2.isHighway);
	return a.CheckForIntersection(b);
}

FVector2D AStreetGenerator::GetIntersectionLocation(FStreetSegment s1, FStreetSegment s2)
{
	Segment a = Segment(s1.start, s1.end, s1.isHighway);
	Segment b = Segment(s2.start, s2.end, s2.isHighway);
	return a.GetIntersectionLocation(b);
}
