// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "QuadTree.h"
#include "Segment.h"
#include "Building.h"
#include "PerlinNoise.h"
#include "ProceduralCityGameModeBase.h"
#include "StreetSegment.h"
#include "BuildingSegment.h"
#include "StreetGenerator.generated.h"

UCLASS()
class PROCEDURALCITY_API AStreetGenerator : public AActor
{
	GENERATED_BODY()
	
public:	
	// Sets default values for this actor's properties
	AStreetGenerator();

protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;
	UFUNCTION(BlueprintCallable)//CallInEditor?
		void Generate();
	UFUNCTION(BlueprintCallable)
		void CreateInitialSetup();
	UFUNCTION(BlueprintCallable)
		TArray<FStreetSegment> GetSegments();
	UFUNCTION(BlueprintCallable)
		TArray<FBuildingSegment> GetBuildings();
	UFUNCTION(BlueprintCallable)
		TArray<FStreetSegment> GetActivSegments();
		bool GetIntersection(FStreetSegment s1, FStreetSegment s2);
		FVector2D GetIntersectionLocation(FStreetSegment s1, FStreetSegment s2);
	UFUNCTION(BlueprintCallable)
		void MakeNoiseP();
	UFUNCTION(BlueprintCallable)
		TArray<FVector2D> GetPoints();
	UFUNCTION(BlueprintCallable)
		TArray<FBuildingSegment> DrawQuadtreeBP();
	UFUNCTION(BlueprintCallable)
		void ConstructCity();

	TArray<Segment> activSegments;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float areaWidth;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite)
		float areaHeight;
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		float shiftX;
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		float shiftY;
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		bool normalizeNoise;
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		bool usePersistance;
	// [0.0f, infinit[
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		float frequenceChange;
	// [0.0f, infinit[
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		float amplitudeChange;
	// [1, infinit[
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		int octaves;
	// ]0.0f, 1.0f]
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		float persistence = 0.5f;
	//commercial, industrial, park, residental, last one allways default
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		TArray<UMaterial*> materials;
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		bool showStreets;
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		bool showBuildings;
	UPROPERTY(EditAnywhere, BlueprintReadWrite)
		bool createNoiseTexture;

private:
	float GetNoiseValue(int x, int y);
	float GetHeight(int x, int y);
	TArray<FVector2D> GetStartpoints();
	Segment SplitSegment(Segment &toSplit, FVector2D splitLocation);
	void UpdateBuildings(Segment newSegment, FVector2D rotDirection);
	void ConstructBuildings(Segment newSegment, FVector2D rotDirection, float height);
	TArray<FBuildingSegment> DrawQuadtree(QuadTree<Building> qT);
	EBuildingType GetBuildingType(FVector2D center);
	float DecideHeight(EBuildingType buildingType, FVector2D center);
	//right vector first to get up vector
	FVector GetNormal(FVector v1, FVector v2);
	TArray<Segment> GlobalGoals(Segment segment);
	bool LocalConstraints(Segment &segment);
	bool AdjustSegment(Segment &segment, bool positiveDirection, bool reset = false);
	void VisualizeBuildings();
	void VisualizeStreets();

	int maxCategory = 2;
	AProceduralCityGameModeBase* config;
	PerlinNoise pN;
	PerlinNoise pNHeight;
	QuadTree<Segment> qT_Street;
	QuadTree<Building> qT_Building;
	TArray<float> noise;
	TArray<float> height;
};
